////////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2001, 2002 by Andrei Alexandrescu
// Permission to use, copy, modify, distribute and sell this software for any 
//     purpose is hereby granted without fee, provided that the above copyright 
//     notice appear in all copies and that both that copyright notice and this 
//     permission notice appear in supporting documentation.
// The author makes no representations about the suitability of this software 
//     for any purpose. It is provided "as is" 
//     without express or implied warranty.
////////////////////////////////////////////////////////////////////////////////

#ifndef VARIANT_INC_
#define VARIANT_INC_

#include <typeinfo>
#include "Visitor.h"
#include "Typelist.h"
#include "static_check.h"

////////////////////////////////////////////////////////////////////////////////
// class template ConfigurableUnion
// Builds a union that contains each type in a typelist
// Usage: ConfigurableUnion<TList> is the very type
////////////////////////////////////////////////////////////////////////////////

template <class U> union ConfigurableUnion;

template <> union ConfigurableUnion<Loki::NullType>
{
};

template <class Head, class Tail> 
union ConfigurableUnion< Loki::Typelist<Head, Tail> >
{
   Head head_;
   ConfigurableUnion<Tail> tail_;
};

////////////////////////////////////////////////////////////////////////////////
// class template MaxSize
// Computes the maximum sizeof for all types in a typelist
// Usage: MaxSize<TList>::result
////////////////////////////////////////////////////////////////////////////////

template <class TList> struct MaxSize;

template <> 
struct MaxSize<Loki::NullType>
{
    enum { result = 0 };
};

template <class Head, class Tail>
struct MaxSize< Loki::Typelist<Head, Tail> >
{
private:
    enum { tailResult = int(MaxSize<Tail>::result) };
public:
    enum { result = sizeof(Head) > int(tailResult)
        ? sizeof(Head)
        : int(tailResult) };
};

////////////////////////////////////////////////////////////////////////////////
// class AlignedPODBase
// Defines a host of protected types used by AlignedPOD (defined later)
// Could be just part of AlignedPOD itself, but making it separate ought to 
// reduce compile times 
////////////////////////////////////////////////////////////////////////////////

class AlignedPODBase
{
protected:
    template <class TList, std::size_t size> struct ComputeAlignBound;

    template <std::size_t size>
    struct ComputeAlignBound<Loki::NullType, size>
    {
        typedef Loki::NullType Result;
    };

    template <class Head, class Tail, std::size_t size>
    struct ComputeAlignBound<Loki::Typelist<Head, Tail>, size>
    {
       typedef typename ComputeAlignBound<Tail, size>::Result TailResult;
       
       typedef typename Loki::Select<
               sizeof(Head) <= size,
               Loki::Typelist<Head, TailResult>,
               TailResult>::Result
           Result;
    };

    template <typename U> struct Structify
    { U dummy_; };
    
    class Unknown;

    typedef TYPELIST_36(
            char,
            short int,
            int,
            long int,
            float,
            double,
            long double,
            char*,
            short int*,
            int*,
            long int*,
            float*,
            double*,
            long double*,
            void*,
            Unknown (*)(Unknown),
            Unknown* Unknown::*,
            Unknown (Unknown::*)(Unknown),
            Structify<char>, 
            Structify<short int>, 
            Structify<int>,
            Structify<long int>,
            Structify<float>,
            Structify<double>,
            Structify<long double>,
            Structify<char*>,
            Structify<short int*>, 
            Structify<int*>,
            Structify<long int*>,
            Structify<float*>,
            Structify<double*>,
            Structify<long double*>,
            Structify<void*>,
            Structify<Unknown (*)(Unknown)>,
            Structify<Unknown* Unknown::*>,
            Structify<Unknown (Unknown::*)(Unknown)>
            )
        TypesOfAllAlignments;
};

////////////////////////////////////////////////////////////////////////////////
// class template AlignedPOD
// Computes the alignment of all types in a typelist
// Usage: ConfigurableUnion<TList> is the very type
////////////////////////////////////////////////////////////////////////////////

template <typename TList>
class AlignedPOD : private AlignedPODBase
{
    enum { maxSize = MaxSize<TList>::result };
    typedef ComputeAlignBound<TypesOfAllAlignments, maxSize>::Result 
        AlignTypes;
public:
    typedef ConfigurableUnion<AlignTypes> Result;
};

////////////////////////////////////////////////////////////////////////////////
// class template MakeConst
// Given a typelist TList, returns a typelist that contains the types in TList 
// adding a const qualifier to each.
// Usage: MakeConst<TList>::Result 
////////////////////////////////////////////////////////////////////////////////

template <class TList> struct MakeConst;

template <> struct MakeConst<Loki::NullType>
{
    typedef Loki::NullType Result; // terminator is not const
};

template <typename Head, class Tail>
struct MakeConst< Loki::Typelist<Head, Tail> >
{
private:
    typedef typename MakeConst<Tail>::Result NewTail;
public:
    typedef Loki::Typelist<const Head, NewTail> Result; 
};

////////////////////////////////////////////////////////////////////////////////
// class template Converter
// Supports the Variant-to-Variant conversion constructor
// Guaranteed to issue an internal compiler error on:
//      1. Metrowerks CodeWarrior 7.0 (internal compiler error: File: 
//          'CTemplateTools.c' Line: 1477 
//          Variant.h line 244           UnitBase > VisitorBase;)
//      2. Microsoft Visual C++ 7.1 alpha release (Assertion failed: 
//          ( name - nameBuf ) < LIMIT_ID_LENGTH, 
//          file f:\vs70builds\2108\vc\Compiler\CxxFE\sl\P1\C\outdname.c, 
//          line 4583)
//      3. GNU gcc 2.95.3-6 (Internal compiler error 980422)
////////////////////////////////////////////////////////////////////////////////

template <class VariantFrom, class VariantTo>
struct Converter
{
    struct UnitBase : public VariantFrom::ConstStrictVisitor
    {
        void* storageForDestination;
    };

    template <class Type, class Base> 
    struct Unit : public Base
    {
        void DoVisit(const Type& obj, Loki::Int2Type<true>)
        {
            new(storageForDestination) VariantTo(obj);
        }
        void DoVisit(const Type&, Loki::Int2Type<false>)
        {
            throw std::runtime_error("Cannot convert");
        }
        virtual void Visit(const Type& obj)
        {
            using namespace Loki;
            enum { dispatch = TL::IndexOf<TList, Type>::value > -1 };
            this->DoVisit(obj, Int2Type<dispatch>());
        }
    };
    
    typedef Loki::GenLinearHierarchy<
        typename VariantFrom::Types,
        Converter::Unit,
        UnitBase > VisitorBase;

    struct Visitor : public VisitorBase
    {
        Visitor(VariantTo& dest)
        {
            // Initialize the pointer to destination
            storageForDestination = &dest;
        }
    };
};

////////////////////////////////////////////////////////////////////////////////
// class template Variant
// Implements a discriminated union in C++
////////////////////////////////////////////////////////////////////////////////

template <class TList, class Align = AlignedPOD<TList>::Result>
class Variant
{
public:
    typedef TList Types;

    // Default constructor
    // Initializes the Variant with a default-constructed object of the 
    // first type in the typelist
    Variant()
    {
        typedef typename TList::Head T;
        new(&buffer_[0]) T;
        vptr_ = &VTableImpl<T>::vTbl_;
    }

    // Copy constructor
    Variant(const Variant& rhs)
    {
        (rhs.vptr_->clone_)(rhs, *this);
    }

    // Converting constructor; accepts any type in the typelist
    // @@@ Suggested simple improvement: accept any type convertible to one of 
    // the types in the typelist. Use Loki::Conversion as a building block
    template <class T>
    explicit Variant(const T& val)
    {
        STATIC_CHECK((Loki::TL::IndexOf<TList, T>::value >= 0), 
            Invalid_Type_Used_As_Initializer);
        
        new(&buffer_[0]) T(val);
        vptr_ = &VTableImpl<T>::vTbl_;
    }
    
    // Inter-Variant conversion constructor
    // Current policy is: conversion succeeds iff the actual type of the source 
    // is one of the types accepted by the target
    // @@@ Possible change: accept if the actual type of the source is
    //     convertible to one of the types accepted by the target. Problem to 
    //     solve: handle ambiguities in a satisfactory manner. Suggestion: when
    //     in doubt, do closest to what the compiler would do.
    template <class TList2, typename Align2>
    explicit Variant(const Variant<TList2, Align2>& rhs)
    {
    	Converter<Variant<TList2, Align2>, Variant>::Visitor v(*this);
        Variant<TList2, Align2>::ConstStrictVisitor& visitor = v;
    	rhs.Accept(visitor);
    }

    // Create a new Variant of the same type as the source, without copying the 
    // destination into it. The newly created object is default constructed.
    enum CloneTypeOnly { cloneTypeOnly };
    Variant(const Variant& rhs, CloneTypeOnly)
    {
        (rhs.vptr_->cloneTypeOnly_)(rhs, *this);
    }

    // Canonic assignment operator
    Variant& operator=(const Variant& rhs)
    {
        Variant temp(rhs);
        temp.Swap(*this);
    }
    
    // Assignment operator from one of the allowed types
    // This is necessary because the constructor is explicit
    template <class T> 
    Variant& operator=(const T& rhs)
    {
        Variant temp(rhs);
        temp.Swap(*this);
    }
    
    // Assignment from another Variant instantiation
    template <class TList2, typename Align2>
    Variant& operator=(const Variant<TList2, Align2>& rhs)
    {
        Variant temp(rhs);
        temp.Swap(*this);
    }
    
    // ~
    ~Variant()
    {
        (vptr_->destroy_)(*this);
    }

    // Visitors definitions
    // @@@ Possible improvement: add defintions of visitor who return
    //     something else than void

    typedef Loki::Visitor<TList, void> StrictVisitor;
    typedef Loki::Visitor<typename MakeConst<TList>::Result, void>
        ConstStrictVisitor;
    typedef Loki::NonStrictVisitor<TList, void> NonStrictVisitor;
    typedef Loki::NonStrictVisitor<typename MakeConst<TList>::Result, void> 
        ConstNonStrictVisitor;

    struct VTable;

    // VTable concrete implementations
    // VTableImpl<T> contains definitions for all of a VTable's pointer to
    // functions.

    template <class T>
    struct VTableImpl
    {
        static const std::type_info& TypeId()
        {
            return typeid(T);
        }
        
        static void Destroy(const Variant& var)
        {
            const T& data = *reinterpret_cast<const T*>(&var.buffer_[0]);
            data.~T();
        }
        
        static void Swap(void* lhs, void* rhs)
        {
            using namespace std;
            swap(*static_cast<T*>(lhs), *static_cast<T*>(rhs));
        }

        static void Clone(const Variant& src, Variant& dest)
        {
            new(&dest.buffer_[0]) T(
                *reinterpret_cast<const T*>(&src.buffer_[0]));
            dest.vptr_ = src.vptr_;
        }
        
        static void CloneTypeOnly(const Variant& src, Variant& dest)
        {
            new(&dest.buffer_[0]) T;
            dest.vptr_ = src.vptr_;
        }
        
        static void Accept(Variant& var, StrictVisitor& visitor)
        {
            visitor.Visit(
                *reinterpret_cast<T*>(&var.buffer_[0]));
        }        
        
        static void Accept(const Variant& var, ConstStrictVisitor& visitor)
        {
            visitor.Visit(
                *reinterpret_cast<const T*>(&var.buffer_[0]));
        }        

        static VTable vTbl_;
    };

    // VTable structure
    // The essential component of the fake vtable idiom, VTable contains
    // pointers to functions, pointers that will be filled up with addresses
    // of functions in VTableImpl

    struct VTable
    {
        const std::type_info& (*typeId_)();
        void (*destroy_)(const Variant&);
        void (*clone_)(const Variant&, Variant&);
        void (*cloneTypeOnly_)(const Variant&, Variant&);
        void (*swap_)(void* lhs, void* rhs);
        void (*accept_)(Variant&, StrictVisitor&);
        void (*acceptConst_)(const Variant&, ConstStrictVisitor&);
    };

public:   // should be private; some compilers prefer 'public' :o}

    enum { neededSize = MaxSize<TList>::result };

    VTable* vptr_;
    union
    {
        Align dummy_;
        char buffer_[neededSize];
    };
    
public:
    void Swap(Variant& rhs)
    {
        // Create a temporary storage
        Variant temp(rhs, cloneTypeOnly);
        // Move rhs into temp
        (rhs.vptr_->swap_)(rhs.buffer_, temp.buffer_);
        rhs.~Variant();
        // Move *this into rhs
        new(&rhs) Variant(*this, cloneTypeOnly);
        (vptr_->swap_)(buffer_, rhs.buffer_);
        this->~Variant();
        // Move temp into *this
        new(this) Variant(temp, cloneTypeOnly);
        (vptr_->swap_)(buffer_, temp.buffer_);
    }
    
    const std::type_info& TypeId() const
    {
        return (vptr_->typeId_)();
    }
    
    template <typename T> T* GetPtr()
    {
        return TypeId() == typeid(T) 
            ? reinterpret_cast<T*>(&buffer_[0])
            : 0;
    }
    
    template <typename T> const T* GetPtr() const
    {
        return TypeId() == typeid(T) 
            ? reinterpret_cast<const T*>(&buffer_[0])
            : 0;
    }
    
    template <typename T> T& Get()
    {
        T* p = GetPtr();
        if (!p) throw std::runtime_error();
        return *p;
    }
    
    template <typename T> const T& Get() const
    {
        const T* p = GetPtr();
        if (!p) throw std::runtime_error();
        return *p;
    }

    // Visitation primitives
    // Although there are four visitor types, only two Accept functions are 
    // necessary, because the non-strict visitors inherit the strict visitors

    void Accept(StrictVisitor& visitor)
    {
        (vptr_->accept_)(*this, visitor);
    }

    void Accept(ConstStrictVisitor& visitor) const
    {
        (vptr_->acceptConst_)(*this, visitor);
    }

    // Extracts the value of a Variant converted to a specific type

    template <class T> T ConvertTo() const;

    // Changes the type of a Variant in-place

    template <class T> void ChangeType();
};

template <class TList, class Align> 
template <class T>
typename Variant<TList, Align>::VTable 
Variant<TList, Align>::VTableImpl<T>::vTbl_ = 
{
    &VTableImpl<T>::TypeId,
    &VTableImpl<T>::Destroy,
    &VTableImpl<T>::Clone,
    &VTableImpl<T>::CloneTypeOnly,
    &VTableImpl<T>::Swap,
    &VTableImpl<T>::Accept,
    &VTableImpl<T>::Accept
};

////////////////////////////////////////////////////////////////////////////////
// class template ConverterTo
// Supports Variant-to-T conversion
////////////////////////////////////////////////////////////////////////////////

template <class VariantFrom, class T>
struct ConverterTo
{
    struct UnitBase : public VariantFrom::ConstStrictVisitor
    {
        T destination_;
    };

    template <class Type, class Base> 
    struct Unit : public Base
    {
        void DoVisit(const Type& obj, Loki::Int2Type<true>)
        {
            destination_ = obj;
        }
        void DoVisit(const Type&, Loki::Int2Type<false>)
        {
            throw std::runtime_error("Cannot convert");
        }
        virtual void Visit(const Type& obj)
        {
            using namespace Loki;
            enum { dispatch = Conversion<Type, T>::exists != 0 };
            this->DoVisit(obj, Int2Type<dispatch>());
        }
    };
    
    typedef Loki::GenLinearHierarchy<
        typename VariantFrom::Types,
        ConverterTo::Unit,
        UnitBase > VisitorBase;

    struct Visitor : public VisitorBase
    {
        Visitor() 
        {
        }
        Visitor(const T& dest) : destination_(dest)
        {
        }
    };
};

template <class TList, class Align> 
template <class To> 
To Variant<TList, Align>::ConvertTo() const
{
    ConverterTo<Variant, To>::Visitor v;
    ConstStrictVisitor& visitor = v;
    this->Accept(visitor);
    return v.destination_;
}

template <class TList, class Align> 
template <class To> 
void Variant<TList, Align>::ChangeType()
{
    Variant(this->ConvertTo<To>()).Swap(*this);
}

#endif
