//	A few routines for setting and getting the values of controls.
//
#include "UniversalSymbols.h"

#if	(UT_TARGET_MACOS)
#include "Controls.h"
#include <Controls.h>
#include <ControlDefinitions.h>

namespace PortableViews {

//@function:Controls::CreatePlatformControl
//@return:platformControl
//The platform control created by the function, or NULL if the control could not be created.

//@arg:type:controlType
//The type of control you wish to create.  Valid values are kAOControlScrollbar, kAOControlCheckbox
//kAOControlButton, or kAOControlRadio.

//@arg:parent:platformWindow
//The window to which this control will be attached.

//@arg:label:constPStringPtr
//The text label for this control.  ScrollBars don't have a label, so you can pass NULL.

//@arg:pos:platformRect*
//The rectangle of the parent window that this control should "fill"

//@arg:commandID:sInt16
//A commandID to identify this control.  What is done with the commandID is system
//or framework dependent.

//@arg:initValue:sInt16
//The initial value (0 or 1 for everything but scrollbars) for this control.  Ignored
// for many control types for which a numeric value makes no sense.

//@arg:isDefault:bool [optional]
// pass "true" if this is the default button, "false" otherwise.  Ignored for non-pushbutton
// controls.

//@arg:resourceRef:sInt16 [optional]
// Pass the resource number for resource-based controls (Pictures and Icons).  Ignored for
// non-resource-based controls.

//@arg:initialText:const PortString const pointer [optional]
// For edit text controls (only), a pointer to a PortString containing the initial text of the control.

//@descr
//Creates any simple control type from the specified arguments.
//@endf
platformControl	CreatePlatformControl(controlType type, platformWindow parent, 
				const PortString& label, platformRect* pos, 
				sInt16 /*commandId*/, sInt16 initValue, bool /*isDefault*/,
				sInt16 resourceRef, const PortString* const initialText)
	{
	sInt16 compMin = 0;
	sInt16 compMax = 1;
	sInt16 procID = pushButProc;
	ControlHandle ctl;
	static long menCount = 0;
	menCount++;
	
	switch (type)
		{
		case kAOControlScrollbar:
			ctl = NewControl(parent, pos, 
								"\p",
								true, 0, 0, 0, kControlScrollBarProc , 0);
			break;
		case kAOControlButton:
			ctl = NewControl(parent, pos, 
								(const unsigned char *)label,
								true, 0, 0, 1, kControlPushButtonProc, 0);
			break;
		case kAOControlRadio:
			ctl = NewControl(parent, pos, 
								(const unsigned char *)label,
								true, 0, 0, 1, kControlRadioButtonProc , 0);
			SetPlatformCheckOrRadioValue(ctl, initValue);
			break;
		case kAOControlCheckbox:
			ctl = NewControl(parent, pos, 
								(const unsigned char *)label,
								true, 0, 0, 1, kControlCheckBoxProc , 0);
			SetPlatformCheckOrRadioValue(ctl, initValue);
			break;
		case kAOControlStaticText:
			ctl = NewControl(parent, pos, 
								"\p",
								true, 0, 0, 1, kControlStaticTextProc , 0);
			SetStaticTextText(ctl, label);
			break;
		case kAOControlEditText:
			ctl = NewControl(parent, pos, 
								"\p",
								true, 0, 0, 1, kControlEditTextProc , 0);
			SetEditTextText(ctl, *initialText);
			break;
		case kAOControlIcon:
			ctl = NewControl(parent, pos, 
								(const unsigned char *)label,
								true, resourceRef, 0, 1, kControlIconRefProc , 0);
			break;
		case kAOControlPicture:
			ctl = NewControl(parent, pos, 
								(const unsigned char *)label,
								true, resourceRef, 0, 1, kControlPictureNoTrackProc , 0);
			break;
		case kAOControlFramedBox:
			ctl = NewControl(parent, pos, 
								(const unsigned char *)label,
								true, 0, 0, 1, kControlGroupBoxTextTitleProc, 0);
			break;
		case kAOControlListBox:
			CreateListBoxControl(parent, pos, true, 0,
										1, false, true, 0,
										0, false,  NULL, &ctl);
			break;
		default:
			// Something we didn't plan on?
			PortAssert(false);
			break;
		}
	return ctl;
	}

//@function:Controls::SetScrollbarControlRange
//@arg:theBar:platformControl
//	The Scrollbar whose range you wish to set.
//@arg:minVal:sInt16
//	The minimum value for this scroll bar (usually 0)
//@arg:maxVal:sInt16
//	The maximum value for this scroll bar.
//@descr
//	Defines the range of values over which the scroll bar can vary.
//@endf
void SetScrollbarControlRange(platformControl theBar, sInt16 minVal, sInt16 maxVal)
	{
	SetControlMinimum(theBar, minVal);
	SetControlMaximum(theBar, maxVal);
	}

//@function:Controls::SetScrollbarControlValue
//@arg:theBar:platformControl
//	The scroll bar whose thumb value (position) you wish to set.
//@arg:theVal:sInt16
//	The position to which you set the thumb.  If this is outside the
//	range of the scroll bar's min and max values, the thumb and value
// 	will actually be set to the closest of (min/max).
//@descr
//	Set the position of the scrollbar's thumb.
//@endf
void SetScrollbarControlValue(platformControl theBar, sInt16 theVal)
	{
	SetControlValue(theBar, theVal);
	}
	
//@function:Controls::DisablePlatformControl
//@arg:theControl:platformControl
//	The control you wish to disable.
//@descr
//	Disables (makes inactive and draws greyed out) the specified control.
//@endf
void	DisablePlatformControl(platformControl theControl)
	{
	HiliteControl(theControl, 255);
	}
	
//@function:Controls::EnablePlatformControl
//@arg:theControl:platformControl
//	The control you wish to enable.
//@descr
//	Re-enables a disabled control.
//@endf
void	EnablePlatformControl(platformControl theControl)
	{
	HiliteControl(theControl, 0);
	}
	
//@function:Controls::HidePlatformControl
//@arg:theControl:platformControl
//	The control you wish to hide.
//@descr
//	Makes the specified control invisible, effectively removing it.
//@endf
void	HidePlatformControl(platformControl theControl)
	{
	HideControl(theControl);
	}
	
//@function:Controls::ShowPlatformControl
//@arg:theControl:platformControl
//	The control you wish to show.
//@descr
//	Makes a previously hidden control reappear.
//@endf
void	ShowPlatformControl(platformControl theControl)
	{
	ShowControl(theControl);
	}

//@function:Controls::GetScrollbarControlValue
//@return:sInt16
//	The current value of the scroll bar thumb.
//@arg:theBar:platformControl
//	The scroll bar whose value you wish to determine.
//@descr
//	Returns the current position of the scrollbar thumb.  This
//	will be between the scrollbar's min and max values, inclusively.
//@endf
sInt16	GetScrollbarControlValue(platformControl theBar)
	{
	return GetControlValue(theBar);
	}

//@function:Controls::GetPlatformCheckOrRadioValue
//@return:sInt16
//	The current value of control: (1) on, or (0) off
//@arg:theControl:platformControl
//	The control whose value you wish to determine.
//@descr
//	Returns the current value of the control, 0 or 1.
//@endf
sInt16	GetPlatformCheckOrRadioValue(platformControl theControl)
	{
	return GetControlValue(theControl);
	}
	
//@function:Controls::SetPlatformCheckOrRadioValue
//@arg:theControl:platformControl
//	The control whose value you wish to set.
//@arg:theValue:sInt16
//	The value to set the control to.  Only 0 or 1 are valid, other values
//	are treated as 1.
//@descr
//	Sets the state of the specified control.
//@endf
void	SetPlatformCheckOrRadioValue(platformControl theControl, sInt16 theValue)
	{
	SetControlValue(theControl, theValue);
	}
	
//@function:Controls::MovePlatformControl
//@arg:theControl:platformControl
//	The control (not scrollbar) whose position you wish to change.
//@arg:newLoc:platformRect*
//	The new rectangle to be filled by the control.
//@descr
//	Moves the control to fill the specified rectangle.
//
//	Windows makes a distinction between scrollbars and other controls, so this
//	function should not be called for a scrollbar.  Instead, call MoveScrollbarControl,
// 	below.
//@endf
void	MovePlatformControl(platformControl theControl, platformRect* newLoc)
	{
	HideControl(theControl);
	MoveControl(theControl, newLoc->left, newLoc->top);
	SizeControl(theControl, newLoc->right - newLoc->left, newLoc->bottom - newLoc->top);
	ShowControl(theControl);
	}
	
//@function:Controls::MoveScrollbarControl
//@arg:theControl:platformControl
//	The scrollbar whose position you wish to change.
//@arg:newLoc:platformRect*
//	The new rectangle to be filled by the scrollbar.
//@descr
//	Moves the scrollbar to fill the specified rectangle.
//@endf
void	MoveScrollbarControl(platformControl theControl, platformRect* newLoc)
	{
	MovePlatformControl(theControl, newLoc);
	}
	
//@function:Controls::SetEditTextText
//@arg:theControl:platformControl
// The edit text control who's text you wish to set
//@arg:theText:const PortString reference
// The text to set it to.
//@descr
// Modifies the text currently displayed in an edit text field.
//@endf	
void	SetEditTextText(platformControl theControl, const PortString& theText)
	{
	SetControlData(theControl, kControlNoPart, kControlEditTextTextTag, theText.Length(), (const char*)theText);
	Draw1Control(theControl);
	}
	
//@function:Controls::GetEditTextText
//@arg:theControl:platformControl
// The edit text control who's text you wish to get
//@arg:theText:PortString reference
// On exit, the text that is in the field,
//@descr
// Retrieves the text currently displayed in an edit text field.
//@endf	
void	GetEditTextText(platformControl theControl,	PortString& theText)
	{
	Size howBig;
	GetControlData(theControl, kControlNoPart, kControlEditTextTextTag, theText[0], NULL, &howBig);
	char buffer[512];
	GetControlData(theControl, kControlNoPart, kControlEditTextTextTag, howBig, buffer, &howBig);
	buffer[howBig] = 0;
	theText = buffer;
	}

//@function:Controls::SetStaticTextText
//@arg:theControl:platformControl
// The static text control who's text you wish to set
//@arg:theText:const PortString reference
// The text to set it to.
//@descr
// Modifies the text currently displayed in an edit text field.
//@endf	
void	SetStaticTextText(platformControl theControl, const PortString& theText)
	{
	SetControlData(theControl, kControlNoPart, kControlStaticTextTextTag, theText.Length(), (const char*)theText);
	Draw1Control(theControl);
	}

	
//@function:Controls::CountListBoxItems
//@arg:theControl:platformControl
//	The list box control whose contents you wish to count
//@return:sInt16
//	The number of items in the list box.
//@descr
//	Returns the number of items in the list box.
//@endf
sInt16	CountListBoxItems(platformControl theList)
	{
	sInt16 returnVal = 0;
	ListHandle listHandle;
	Size tossme;
	Rect bounds;
	GetControlData(theList, kControlEntireControl, kControlListBoxListHandleTag, sizeof(listHandle), &listHandle, &tossme);
	GetListDataBounds(listHandle, &bounds);
	returnVal = bounds.bottom;
	return returnVal;
	}
	
//@function:Controls::GetListBoxSelection
//@arg:theControl:platformControl
//	The list box control whose selection you wish to find
//@return:sInt16
//	The zero-based index of the selected item.
//@descr
//	Returns the index of the current list box selection.
//@endf
sInt16	GetListBoxSelection(platformControl theList)
	{
	sInt16 returnVal = 0;
	ListHandle listHandle;
	Size tossme;
	GetControlData(theList, kControlEntireControl, kControlListBoxListHandleTag, sizeof(listHandle), &listHandle, &tossme);
	Cell theCell;
	theCell.v = 0;
	theCell.h = 0;
	bool anySelect = LGetSelect(true, &theCell, listHandle);
	if (anySelect)
		returnVal=theCell.v;
	else
		returnVal = kAONoSelection;
	return returnVal;
	}
	
//@function:Controls::GetTextOfListBoxItem
//@arg:theControl:platformControl
//	The list box control
//@arg:theIndex:sInt16
//  The (zero-based) index of the item you wish to examine.
//@return:PortString
//	The text of the specified item
//@descr
//	Returns the text of the specified list box item (which may or may not be
//  the selected item).
//@endf
PortString	GetTextOfListBoxItem(platformControl theList, sInt16 theIndex)
	{
	char buffer[256];
	PortString returnVal;
	ListHandle listHandle;
	Size tossme;
	GetControlData(theList, kControlEntireControl, kControlListBoxListHandleTag, sizeof(listHandle), &listHandle, &tossme);
	Cell theCell;
	theCell.v = theIndex;
	theCell.h = 0;
	sInt16 dataLen = 255;
	LGetCell(buffer, &dataLen, theCell, listHandle);
	buffer[dataLen] = 0;
	returnVal = buffer;						
	return returnVal;
	}
	
//@function:Controls::DeleteListBoxItem
//@arg:theControl:platformControl
//	The list box control
//@arg:theIndex:sInt16
//  The (zero-based) index of the item you wish to delete.
//@descr
//	Deleted the specified item.
//@endf
void	DeleteListBoxItem(platformControl theList, sInt16 theIndex)
	{
	ListHandle listHandle;
	Size tossme;
	GetControlData(theList, kControlEntireControl, kControlListBoxListHandleTag, sizeof(listHandle), &listHandle, &tossme);
	LDelRow(1, theIndex, listHandle);
	Cell theCell;
	theCell.v = theIndex;
	theCell.h = 0;
	LSetSelect(true, theCell, listHandle);
	Draw1Control(theList);
	}
	
//@function:Controls::AddListBoxItem
//@arg:theControl:platformControl
//	The list box control
//@arg:theIndex:sInt16
//  The (zero-based) index of the item you wish to insert after.
//@arg:theItem:PortString reference
//  The text of the new item.
//@descr
//	Adds the specified item.
//@endf
void	AddListBoxItem(platformControl theList, sInt16 atPos, const PortString& theItem)
	{
	ListHandle listHandle;
	Size tossme;
	GetControlData(theList, kControlEntireControl, kControlListBoxListHandleTag, sizeof(listHandle), &listHandle, &tossme);
	LAddRow(1, atPos, listHandle);
	Cell theCell;
	theCell.v = atPos;
	theCell.h = 0;
	LSetCell((const char*)theItem, theItem.Length(), theCell, listHandle);
	Draw1Control(theList);
	}

//@function:Controls::SelectListBoxItem
//@arg:theControl:platformControl
//	The list box control
//@arg:theIndex:sInt16
//  The (zero-based) index of the item you wish to select in the box.  Any previous
// selection will be cleared.
//@descr
//	Selects the the specified item.
//@endf
void	SelectListBoxItem(platformControl theList, sInt16 theIndex)
	{
	ListHandle listHandle;
	Size tossme;
	GetControlData(theList, kControlEntireControl, kControlListBoxListHandleTag, sizeof(listHandle), &listHandle, &tossme);
	Cell theCell;
	theCell.v = 0;
	theCell.h = 0;
	bool anySelect = LGetSelect(true, &theCell, listHandle);
	if (anySelect)
		LSetSelect(false, theCell, listHandle);
	theCell.v = theIndex;
	theCell.h = 0;
	LSetSelect(true, theCell, listHandle);
	Draw1Control(theList);
	}
	
} // namespace PortableViews
#endif
