#include "UniversalSymbols.h"
#include "Dialog.h"
#include "DemoDialogs.h"

using namespace PortableViews;


// Some helper functions.  Note that in a real application
// an XML parser would generate these structures directly,
// so these routines wouldn't be necessary.

// The routines DemoDialog1(), DemoDialog2(), and DemoDialog3()
// show how these dialogs would actually be used in the presence
// of an external (XML?) loader, the routines BuildDialog1(),
// BuildDialog2(), and BuildDialog3() are specific shims that
// would be replaced with a general parser that built the
// correct structures.

// Fill in an PortDialogInfo record
inline void InitInfo(PortDialogInfo& sample,
				const PortString& title, 
				LayoutNode& node)
	{
	sample.iTitle = title;
	sample.iWidth = 0;
	sample.iHeight = 0;
	sample.iLayoutInfo = node;
	sample.iLayoutUsed = true;
	}
	

// Create a control
inline void InitControl(LayoutControlList& list,
				LayoutNode& node,
				PortString title,
				LayoutControlType type,
				sInt16 commandID,
				bool   isDefault = false,
				sInt16 forceWidth = 0)
	{
	if (type != kLayoutFill && type != kLayoutSpace)
		{
		LayoutControl ctl;
		ctl.iType = type;
		ctl.iCommandID = commandID;
		ctl.iDefault = isDefault;
		ctl.iResourceRef = 0;
		SetRectTLBR(&ctl.iRect, 0, 0, 0, 0);
		if (forceWidth != 0)
			{
			ctl.iFixedWidth = true;
			ctl.iRect.left = 0;
			ctl.iRect.right = forceWidth;
			}
		else
			ctl.iFixedWidth = false;
		ctl.iFixedHeight = false;
		ctl.iCancel = false;
		ctl.iCloses = (type == kAOControlButton); // All buttons will close
		ctl.iRadioGroup = 1;
		ctl.iTextValue = title; // This puts the title text into edit text boxes.
		ctl.iNumericValue = isDefault ? 1 : 0;
		ctl.iTitle = title;
		list.push_back(ctl);
		node.control = commandID;
		}
	else
		node.control = type;
	node.x = 0;
	node.y = 0;
	node.width = 0;
	node.height = 0;
	}
	
// Create a box
inline void InitBox(LayoutNode& node, bool horizontal, bool framed, PortString title)
	{
	node.control = (horizontal) ? kLayoutHBox : kLayoutVBox;
	node.framed = framed;
	node.label = title;
	node.x = 0;
	node.y = 0;
	node.width = 0;
	node.height = 0;
	}
				


// We'll begin with a very simple dialog, just
// asking the user a question, with a place for
// them to enter data, and OK/Cancel buttons.

// In XML, this would look like this:
// <!-- The simple input dialog-->
// <DIALOG id="500" title="Simple Input">
//	<LAYOUT>
//	  <VBOX>
//	    <STATICTEXT id="102" width="200">A short text</STATICTEXT>
//	    <EDITTEXT id="103" width="200>Hello World</EDITTEXT>
//	    <HBOX>
//	      <FILL/>
//	      <BUTTON id="100" default="yes">OK</BUTTON>
//	      <BUTTON id="101" cancel="yes">Cancel</BUTTON>
//	      <FILL aquaonly="yes"/>
//	    </HBOX>
// 	  </VBOX>
// 	</LAYOUT>
// </DIALOG>
void BuildDialog1(LayoutControlList& theList, PortDialogInfo& theInfo)
	{
	
	// One fill object, we can copy it as often as we need.
	LayoutNode theFill;
	InitControl(theList, theFill, PortString(""), kLayoutFill, 0, false);

	
	// A static string (forcing width to 200 to limit the second display.)
	LayoutNode statString;
	InitControl(theList, statString, PortString("A short text."), kAOControlStaticText,
					102, false, 200);
	
	// An edit string (have to force width)
	LayoutNode editString;
	InitControl(theList, editString, PortString("Hello"), kAOControlEditText, 103, false, 200);
	
	// The buttons.
	LayoutNode okButton;
	InitControl(theList, okButton, PortString("OK"), kAOControlButton, 100, true);
	LayoutNode cancelButton;
	InitControl(theList, cancelButton, PortString("Cancel"), kAOControlButton, 101, false);
	LayoutNode buttonBox;
	InitBox(buttonBox, true, false, PortString(""));
	buttonBox.children.push_back(theFill);
	buttonBox.children.push_back(okButton);
	buttonBox.children.push_back(cancelButton);
#if (UT_TARGET_MACOS)
	if (IsAquaAppearance())
		buttonBox.children.push_back(theFill);
#endif
	
	// And a box for them all to go in.
	LayoutNode totalBox;
	InitBox(totalBox, false, false, PortString(""));
	totalBox.children.push_back(statString);
	totalBox.children.push_back(editString);
	totalBox.children.push_back(buttonBox);
	
	InitInfo(theInfo, "String Dialog", totalBox);
	}
	
	
// Here's that "Shirt Size" dialog that we keep showing in the text.
// From a fictitious on-line catalog system.
// <!-- The choose shirt size dialog-->
// <DIALOG id="1000" title="Choose Size">
// 	<LAYOUT>
// 	  <VBOX>
// 	    <STATICTEXT id="100">Choose shirt size:</STATICTEXT>
// 	    <HBOX framed="yes" title="Available Sizes>
// 	      <RADIO id="101" group="1" checked="yes">S</RADIO>
// 	      <RADIO id="102" group="1">M</RADIO>
// 	      <RADIO id="103" group="1">L</RADIO>
// 	      <RADIO id="104" group="1">XL</RADIO>
// 	      <RADIO id="105" group="1">XXL</RADIO>
// 	    </HBOX>
// 	    <HBOX>
// 	      <FILL/>
// 	      <BUTTON id="200" default="yes">OK</BUTTON>
// 	      <BUTTON id="201" cancel="yes">Cancel</BUTTON>
// 	      <FILL aquaonly="yes"/>
// 	    </HBOX>
// 	  </VBOX>
// 	</LAYOUT>
// </DIALOG>
void BuildDialog2(LayoutControlList& theList, PortDialogInfo& theInfo)
	{	
	// One fill object, we can copy it as often as we need.
	LayoutNode theFill;
	InitControl(theList, theFill, PortString(""), kLayoutFill, 0, false);

	// The "whole dialog" VBOX
	LayoutNode totalBox;
	InitBox(totalBox, false, false, PortString(""));
	
	// A static string 
	LayoutNode statString;
	InitControl(theList, statString, PortString("Choose Shirt Size"), kAOControlStaticText,
					100, false, 200);
	totalBox.children.push_back(statString);
	
	// Then the radio buttons' box
	LayoutNode rbbox;
	InitBox(rbbox, true, true, PortString("Available Sizes:"));
	
	// Then the buttons themselves
	LayoutNode b1, b2, b3, b4, b5;
	InitControl(theList, b1, PortString("S"), kAOControlRadio, 101, true);
	InitControl(theList, b2, PortString("M"), kAOControlRadio, 102, false);
	InitControl(theList, b3, PortString("L"), kAOControlRadio, 103, false);
	InitControl(theList, b4, PortString("XL"), kAOControlRadio, 104, false);
	InitControl(theList, b5, PortString("XXL"), kAOControlRadio, 105, false);
	rbbox.children.push_back(b1);
	rbbox.children.push_back(b2);
	rbbox.children.push_back(b3);
	rbbox.children.push_back(b4);
	rbbox.children.push_back(b5);
	totalBox.children.push_back(rbbox);
		
	// The buttons.
	LayoutNode buttonBox;
	InitBox(buttonBox, true, false, PortString(""));
	LayoutNode okButton;
	InitControl(theList, okButton, PortString("OK"), kAOControlButton, 200, true);
	LayoutNode cancelButton;
	InitControl(theList, cancelButton, PortString("Cancel"), kAOControlButton, 201, false);
	buttonBox.children.push_back(theFill);
	buttonBox.children.push_back(okButton);
	buttonBox.children.push_back(cancelButton);
#if (UT_TARGET_MACOS)
	if (IsAquaAppearance())
		buttonBox.children.push_back(theFill);
#endif
	totalBox.children.push_back(buttonBox);
	
	InitInfo(theInfo, "Shirt Size Selection", totalBox);
	}
	

// And a complicated example to end up with:
// <!-- A "Create Project" Dialog -->
// <DIALOG id="1001" title="Create New Project">
// 	<LAYOUT>
// 		<VBOX>
// 			<VBOX framed="Project Information">
// 				<HBOX>
// 					<STATICSTRING id="4">Project Name:</STATICSTRING>
// 					<EDITTEXT id="7" width="150">Untitled Project</EDITTEXT>
// 				</HBOX>
// 				<HBOX>
// 					<STATICSTRING id="5">Short Name (no spaces):</STATICSTRING>
// 					<EDITTEXT id="8" width="150">Untitled</EDITTEXT>
// 				</HBOX>
// 				<HBOX>
// 					<STATICSTRING id="9">Application Type:</STATICSTRING>
// 					<LISTBOX id="10" width="250">
// 						<LISTITEM string="No Windows/No Docs"/>
// 						<LISTITEM string="Non Document"/>
// 						<LISTITEM string="Single Document"/>
// 						<LISTITEM string="Multi Document"/>
// 						<LISTITEM string="Multi Document (Old MDI)"/>
// 						<LISTITEM string="Fullscreen"/>
// 					</LISTBOX>
// 				</HBOX>
// 				<CHECKBOX id="14" value="1">Use OpenGL</CHECKBOX>
// 				<CHECKBOX id="15" value="0">Use QuickTime</CHECKBOX>
// 				<CHECKBOX id="6" value="1">Create Folder</CHECKBOX>				
// 			</VBOX>
// 			<HBOX>
// 				<FILL/>
// 				<BUTTON id="1" default="yes" close="yes">Create Project</BUTTON>
// 				<BUTTON id="2" cancel="yes" close="yes">Cancel</BUTTON>
// 	     		<FILL aquaonly="yes"/>
// 			</HBOX>
// 		</VBOX>
// 	</LAYOUT>
// </DIALOG>
void BuildDialog3(LayoutControlList& theList, PortDialogInfo& theInfo)
	{	
	// One fill object, we can copy it as often as we need.
	LayoutNode theFill;
	InitControl(theList, theFill, PortString(""), kLayoutFill, 0, false);

	// The "whole dialog" VBOX
	LayoutNode totalBox;
	InitBox(totalBox, false, false, PortString(""));
	
	// The "Project Information" VBOX
	LayoutNode prInfo;
	InitBox(prInfo, false, true, PortString("Project Information"));
	
	// Project Name HBOX
	LayoutNode prName;
	InitBox(prName, true, false, PortString(""));
	
	// A static string 
	LayoutNode statString1;
	InitControl(theList, statString1, PortString("Project Name"), kAOControlStaticText,
					4, false);
	prName.children.push_back(statString1);
	
	// An Edit String
	LayoutNode editString1;
	InitControl(theList, editString1, PortString("Untitled Project"), kAOControlEditText,
					7, false, 150);
	prName.children.push_back(editString1);
	prInfo.children.push_back(prName);
	
	// Short name HBOX
	// Project Name HBOX
	LayoutNode prShortName;
	InitBox(prShortName, true, false, PortString(""));
	
	// A static string 
	LayoutNode statString2;
	InitControl(theList, statString2, PortString("Short Name"), kAOControlStaticText,
					5, false);
	prShortName.children.push_back(statString2);
	
	// An Edit String
	LayoutNode editString2;
	InitControl(theList, editString2, PortString("Project"), kAOControlEditText,
					8, false, 150);
	prShortName.children.push_back(editString2);
	prInfo.children.push_back(prShortName);
	
	// Then the radio buttons' box
	LayoutNode rbbox;
	InitBox(rbbox, false, true, PortString("Application Type:"));
	
	// Then the radio buttons themselves
	LayoutNode b1, b2, b3, b4, b5, b6;
	InitControl(theList, b1, PortString("No Windows/No Docs"), kAOControlRadio, 20, true);
	InitControl(theList, b2, PortString("Non Document"), kAOControlRadio, 21, false);
	InitControl(theList, b3, PortString("Single Document"), kAOControlRadio, 22, false);
	InitControl(theList, b4, PortString("Multi Document"), kAOControlRadio, 23, false);
	InitControl(theList, b5, PortString("Multi Document (Old MDI Style)"), kAOControlRadio, 24, false);
	InitControl(theList, b6, PortString("Fullscreen"), kAOControlRadio, 25, false);
	rbbox.children.push_back(b1);
	rbbox.children.push_back(b2);
	rbbox.children.push_back(b3);
	rbbox.children.push_back(b4);
	rbbox.children.push_back(b5);
	rbbox.children.push_back(b6);
	prInfo.children.push_back(rbbox);
	
	// A few checboxes
	LayoutNode c1, c2, c3;
	InitControl(theList, c1, PortString("Use OpenGL"), kAOControlCheckbox, 14, true);
	InitControl(theList, c2, PortString("Use QuickTime"), kAOControlCheckbox, 15, false);
	InitControl(theList, c3, PortString("Create Folder"), kAOControlCheckbox, 6, true);
	prInfo.children.push_back(c1);
	prInfo.children.push_back(c2);
	prInfo.children.push_back(c3);
	totalBox.children.push_back(prInfo);
	
	// Finally,the bottons..
	LayoutNode buttonBox;
	InitBox(buttonBox, true, false, PortString(""));
	LayoutNode okButton;
	InitControl(theList, okButton, PortString("OK"), kAOControlButton, 200, true);
	LayoutNode cancelButton;
	InitControl(theList, cancelButton, PortString("Cancel"), kAOControlButton, 201, false);
	buttonBox.children.push_back(theFill);
	buttonBox.children.push_back(okButton);
	buttonBox.children.push_back(cancelButton);
#if (UT_TARGET_MACOS)
	if (IsAquaAppearance())
		buttonBox.children.push_back(theFill);
#endif
	totalBox.children.push_back(buttonBox);
	
	InitInfo(theInfo, "Create New Project", totalBox);
	}
	

void DemoDialogOne(void)
	{
	LayoutControlList theList;
	PortDialogInfo info;

	// Load the dialog from hypothetical "XML" reader	
	BuildDialog1(theList, info);
	
	// And display it
	PortDialog dlg(theList, info);
	dlg.DoModal();
	
	// Then display it again, after putting a much longer string into the static text 
	// box.
	theList[0].iTitle = PortString("This is the same dialog, with a much greater amount");
	theList[0].iTitle += PortString(" of text added to it.  Because static text controls");
	theList[0].iTitle += PortString(" on the mac are limited to 255 chars, so are we, but");
	theList[0].iTitle += PortString(" that would be an awful lot of text for one dialog, anyway");
	dlg.SetControlList(theList);
	dlg.DoModal();
	}


void DemoDialogTwo(void)
	{
	LayoutControlList theList;
	PortDialogInfo info;

	// Load the dialog from hypothetical "XML" reader	
	BuildDialog2(theList, info);
	
	// And display it
	PortDialog dlg(theList, info);
	dlg.DoModal();
	}

void DemoDialogThree(void)
	{
	LayoutControlList theList;
	PortDialogInfo info;

	// Load the dialog from hypothetical "XML" reader	
	BuildDialog3(theList, info);
	
	// And display it
	PortDialog dlg(theList, info);
	dlg.DoModal();
	}




// Just diplay our dialogs, one right after another.
void DemoDialogs(void)
	{
	DemoDialogOne();
	DemoDialogTwo();
	DemoDialogThree();
	}