// Mac implementation of the PortDialog

#include "UniversalSymbols.h"

#if (UT_TARGET_MACOS)
#include <Dialogs.h>
#include <Sound.h>
#include <Controls.h>
#include <ControlDefinitions.h>
#include <cstring>
#include <CarbonEvents.h>

#include "Controls.h"
#include "Dialog.h"
#include "Window.h"

namespace PortableViews {
	
PortDialog::PortDialog(LayoutControlList& controls, PortDialogInfo& info)
	{
	iResID = 0;
	iDefault = -1;
	iControlList = controls;
	iInfo = info;
	}
	
//
//	Name:	DoModal
//
//	Descr:	Display the dialog as a modal dialog.  Doesn't return until
//		processing is complete and the dialog has gone away.
//
PortDialogItem 	PortDialog::DoModal(void)
	{
	WindowRef dlgWindow = PortWindow::CreatePlatformDialog();
	
	// First, layout the dialog.
	if (iInfo.iLayoutUsed)
		LayoutControls(iInfo.iLayoutInfo);
	long n = iControlList.size();
	PortAssert(n > 0); // Must have a control list, no?
		
	// Center on the screen containing the front window
	WindowRef frWindow = FrontWindow();
	Rect bRect;
	// GWGAD was added in Carbonlib 1.1, so it might not be there...
	if (frWindow != NULL && GetWindowGreatestAreaDevice != NULL)
		{
		GetWindowGreatestAreaDevice(frWindow, kWindowContentRgn, NULL, &bRect);
		}
	else
		{
		// OK, center on the main screen, then.
		GDHandle mainDev = GetMainDevice();
		bRect = (**mainDev).gdRect;
		}
	
	Rect dRect;
	dRect.left = ((bRect.right - bRect.left) - iInfo.iWidth) / 2;
	dRect.top = ((bRect.bottom - bRect.top) - iInfo.iHeight) / 2;
	dRect.right = dRect.left + iInfo.iWidth;
	dRect.bottom = dRect.top + iInfo.iHeight;
	
	MoveWindow(dlgWindow, dRect.left, dRect.top, true);
	SizeWindow(dlgWindow, dRect.right - dRect.left, dRect.bottom - dRect.top, true);
	
	SetWRefCon(dlgWindow, reinterpret_cast<long>((PortWindow*)this));
	SetThemeWindowBackground(dlgWindow, kThemeBrushDialogBackgroundActive, true);
	iWindow = dlgWindow;
	PortWindow::sWindowMap[dlgWindow] = (PortWindow*)this;
	iExiting = false;
	
	// Set some defaults for the buttons
	SetDefaultButton(0);
	SetCancelButton(0);
	
	CreateActualControls();
	for (long i = 0; i < iControlList.size(); i++)
		{
		if (iControlList[i].iDefault && iControlList[i].iType == kAOControlButton)
				SetDefaultButton(iControlList[i].iCommandID);
			if (iControlList[i].iCancel && iControlList[i].iType == kAOControlButton)
				SetCancelButton(iControlList[i].iCommandID);
		}
		
	// Call InitialSetup, which lets the user override the control list settings
	// if they want to (say, for different defaults on subsequent calls.)
	InitialSetup();
	short itemHit = 0;
	
	ShowWindow(iWindow);
	DrawControls(iWindow);
	RunAppModalLoopForWindow(iWindow);

	DisposeWindow(iWindow);
	PortWindow::sWindowMap.erase(dlgWindow);
	
	return iExitValue;
	}
	
	
void	PortDialog::InitialSetup(void)
	{
	// For mac, done elsewhere.
	}

// Process an item hit.  For new-style dialogs, handles the basic behaviors for you.
// In New model dialogs, "theItem" is the command code of the "hit" item.  In old-model
// (non-control-list ones), it's the index into the DITL + 1000.  The old way is deprecated, big-time.
bool	PortDialog::ItemHit(PortDialogItem theItem)
	{
	bool returnVal = false;
	long n = iControlList.size();
	if (n != 0)
		{
		// OK, we're a Control-List dialog.  So we're reqponsible for basic handling (they can override
		// this method if they don't like that.)
		sInt16 index = GetControlListIndexFromCommand(theItem);
		
		// Deal with radio groups.
		if ((index != -1) && iControlList[index].iType == kAOControlRadio)
			{
			// Set everything in our group to false, except us.
			for (long i = 0; i < n; i++)
				{
				if (iControlList[i].iType == kAOControlRadio)
					{
					if (iControlList[i].iRadioGroup == iControlList[index].iRadioGroup)
						{
						platformControl ctrl = GetLayoutControl(iControlList[i].iCommandID);
						if (i == index)
							SetPlatformCheckOrRadioValue(ctrl, 1);
						else
							SetPlatformCheckOrRadioValue(ctrl, 0);
						}
					}
				}
			
			}
			
		// And  checkboxes.
		if ((index != -1) && iControlList[index].iType == kAOControlCheckbox)
			{
			platformControl ctrl = GetLayoutControl(iControlList[index].iCommandID);
			if (GetPlatformCheckOrRadioValue(ctrl) == 0)
				SetPlatformCheckOrRadioValue(ctrl, 1);
			else
				SetPlatformCheckOrRadioValue(ctrl, 0);
			}
			
		// They may have special instructions for us.  Don't validate if 
		// cancel, otherwise true.  Might let them specify in XML someday.
		if ((index != -1) && iControlList[index].iCloses)
			CloseDialog(theItem, !iControlList[index].iCancel);
		}
	return returnVal;
	}
	
//
//	Name:	PortDialog::CloseDialog
//
//	Descr:
//	The dialog should end.  The "onlyIfValid" flag means that the user wants
//	validation first, and if the data isn't valid, the "close" should be cancelled.
//	onlyIfValid defaults to TRUE, it should be passes as FALSE for cancel buttons
//	and other circumstances where you don't care if the data's valid.
//
void	PortDialog::CloseDialog(PortDialogItem result, bool onlyIfValid)
	{
	if ((!onlyIfValid) || ValidData())
		{
		iExiting = true;
		iExitValue = result;
		}
	if (iExiting)
		{
	
		long n = iControlList.size();
		if (n > 0)
			{
			// Store the results of all the controls before we die.
			for (long i = 0; i < n; i++)
				{
				platformControl ctrl = GetLayoutControl(iControlList[i].iCommandID);
				switch (iControlList[i].iType)
					{
					case kAOControlRadio:
					case kAOControlCheckbox:
						{
						iControlList[i].iNumericValue = GetPlatformCheckOrRadioValue(ctrl);
						}
						break;
					case kAOControlEditText:
						GetEditTextText(ctrl, iControlList[i].iTextValue);
						break;
					case kAOControlListBox:
						// Numeric value is zero-based index of list
						// Text value is text of item
						ListHandle listHandle;
						Size tossme;
						GetControlData(ctrl, kControlEntireControl, kControlListBoxListHandleTag, sizeof(listHandle), &listHandle, &tossme);
						Cell theCell;
						theCell.v = 0;
						theCell.h = 0;
						bool anySelect = LGetSelect(true, &theCell, listHandle);
						PortAssert(anySelect);
						iControlList[i].iNumericValue=theCell.v;
						char buffer[256];
						sInt16 dataLen = 255;
						LGetCell(buffer, &dataLen, theCell, listHandle);
						buffer[dataLen] = 0;
						iControlList[i].iTextValue = buffer;						
						break;
					case kAOControlPulldown:
						// Numeric value is zero-based index of menu
						// Text value is text of item
						MenuHandle men;
						Size resultSize;
						GetControlData(ctrl, kControlEntireControl, kControlPopupButtonMenuHandleTag, sizeof(men), &men, &resultSize);
						iControlList[i].iNumericValue = GetControlValue(ctrl) - 1;
						Str255 buff2;
						GetMenuItemText(men, iControlList[i].iNumericValue + 1, buff2);
						iControlList[i].iTextValue=buff2;
						break;
					default:
						// Presently, no other controls have values.
						break;
					}
				}
			}
		
		// Give them their last look at the dialog
		LastChance(result);
	
		// And end our event loop
		QuitAppModalLoopForWindow(iWindow);
		}
	}


} // namespace PortableViews;

#endif
