//
//	PortDialogWin.cpp

// Windows implementation of PortDialog

#include "UniversalSymbols.h"

#if (UT_TARGET_WIN32)
#include <Windows.h>
#include "Dialog.h"
#include "Controls.h"

namespace PortableViews {

//
// Okay, we've got a little problem, which is how to identify the PortDialog from
// the HWND pointer we get passed in.  WIth windows, we solve this by putting the
// AVWINDOW* in the userdata of the window, but that solution won't work here because
// the DialogBox() call doesn't give us the dialog handle--it's created and used internally.
//
//	So we'll cheat.  Just before calling DialogBox(), we'll set a module-global variable
//	(pendingDialog) to the value of the current PortDialog, and in the first dialog message,
//  we'll (a) let the PortDialog know what it's window is, and (b) store the CPDDialog in
//  the userdata of the dialog window.
//  
PortDialog* pendingDialog = NULL;

INT_PTR	CALLBACK	stdDialogProc(HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam);
INT_PTR	CALLBACK	stdDialogProc(HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM /*lParam*/)
	{
	if (pendingDialog != NULL)
		{
		SetWindowLong(hDlg, GWL_USERDATA, reinterpret_cast<long>((PortDialog*)pendingDialog));
		pendingDialog->SetWindow(hDlg);
		pendingDialog = NULL;
		}
	PortDialog* theDialog = reinterpret_cast<PortDialog*>(GetWindowLong(hDlg, GWL_USERDATA));
	bool returnVal = false;
	switch (iMsg)
		{
		case 	WM_INITDIALOG:
			theDialog->SetupControls();
			theDialog->InitialSetup();
			PortWindow::sWindowMap[hDlg] = (PortWindow*)theDialog;
			returnVal = true;
			break;
			
		case 	WM_DESTROY:
			PortWindow::sWindowMap.erase(hDlg);
			break;
			
		case 	WM_COMMAND:
			returnVal =  theDialog->ItemHit(LOWORD(wParam));
			break;
		}
	return returnVal;
	}


//@method:PortDialog::PortDialog
//@arg:controls:LayoutControlList reference
//	The control list for the dialog.
//@arg:info:PortDialogInfo references
//	The Dialog info record for the dialog
//@descr
//	This constructor loads a dialog from a specified control list and dialog info
//  record.  These can be constructed by hand, or returned from
//  AVXMLDialogManager::GetDialogSettings().
//@endm
PortDialog::PortDialog(LayoutControlList& controls, PortDialogInfo& info)
	{
	iResID = 0;
	iDefault = -1;
	iControlList = controls;
	iInfo = info;
	
	iWindow = NULL;
	iUserData = 0;
	iIsDialog = true;
	}
			
//@method:PortDialog::DoModal
//@return:PortDialogItem
//	The dialog item the user clicked on to exit the dialog (by control ID).
//@descr
//	Causes the dialog object to be loaded, laid out (if necessary), displayed,
//  executed, and the id value  of the "closing" control returned.
//  Note that the most of the methods of PortDialog can only be called during
//  the execution of this function (usually by way of an override), but the values
//  of the various fields and controls are stored in the iControlList at the
//  termination of this function, and can be recovered from there.   
//@endm
PortDialogItem 	PortDialog::DoModal(void)
	{
	pendingDialog = this;
	INT_PTR itm;
	long clSize = iControlList.size();
	// First, layout the dialog.
	if (iInfo.iLayoutUsed)
		LayoutControls(iInfo.iLayoutInfo);
	clSize = iControlList.size();
	
	int xsz = GetSystemMetrics(SM_CXSCREEN);
	int ysz = GetSystemMetrics(SM_CYSCREEN);
	uInt16 lpos = (xsz - (iInfo.iWidth)) / 4;
	uInt16 tpos = (ysz - (iInfo.iHeight)) / 4;
	
	// We need a big hunk of memory here for an in-memory dialogs template.
	// Needs enough space to hold a DLGTEMPLATE record, plus up to 2000 bytes for a title
	// string, plus a couple bytes each for a menu, dialog class, and index to number of characters.
	// We'll add a little extra space in case the user overwrites the string or we decide to
	// bump this to a DLGTEMPLATEEX in the future.
	char templateBytes[sizeof(DLGTEMPLATE) + 2000 + 6 + 200];  // 200 pad bytes for safety
	
	char* dtemplate = templateBytes;
	wchar_t wideCharOut[1000];
	// Write a dialog template header
	// It's not documented anywhere, but the resource-based dialog manager adds
	// SS_GRAYRECT to the dialog box; if we don't do it we won't match the system background.
	// Also, DS_CENTER appears not to do anything in this context, so we have to manually
	// specify the position.
	*((uInt32 *)(dtemplate)) = (DS_MODALFRAME | DS_CENTER | WS_POPUP | WS_CAPTION | SS_GRAYRECT); //Style
	dtemplate += 4;
	*((uInt32 *)(dtemplate)) = 0; // Extended Style
	dtemplate += 4;
	*((uInt16 *)(dtemplate)) = 0; // # controls
	dtemplate += 2;
	*((uInt16 *)(dtemplate)) =lpos; // Left (Dialog Box Units)
	dtemplate += 2;
	*((uInt16 *)(dtemplate)) =tpos; // Top
	dtemplate += 2;
	*((uInt16 *)(dtemplate)) =(uInt16)(iInfo.iWidth/2); // Width
	dtemplate += 2;
	*((uInt16 *)(dtemplate)) =(uInt16)(iInfo.iHeight/2); // Height
	dtemplate += 2;
	*((uInt16 *)(dtemplate)) =0; // Menu
	dtemplate += 2;
	*((uInt16 *)(dtemplate)) =0; // Standard Dialog Class
	dtemplate += 2;
	uInt32 howMany = MultiByteToWideChar(CP_ACP, 0, 
				(char *)iInfo.iTitle, -1, wideCharOut, 1000);
	howMany *= 2;
	memcpy(dtemplate, wideCharOut, howMany);
	dtemplate += howMany;
	// Item templates should to go here, but we're going to make them manually at InitialSetup() time.
	*((uInt16 *)(dtemplate)) =0; // No item templates
	dtemplate += 2;
	itm = DialogBoxIndirect(gAppInstance, (const DLGTEMPLATE *)templateBytes, gAppWindow, stdDialogProc);
	return itm;
	}

	
//@method:PortDialog::InitialSetup
//@descr
//	Called by DoModal() after the dialog is created and laid out, but before
//  and user interaction has occurred.  You can override this to change control values,
//  highlight items, set defaults, etc. The size of controls for layout purposes has
//  already occurred by the time this routine is called, so 
//  if the layout of the dialog depends on the size of controls you
//  are setting at run-time, this is too late to do it.  In that instance, modify the
//  values of the control list before calling DoModal().
//@endm
void		PortDialog::InitialSetup(void)
	{
	}


// Internal only, used to setup controls so that InitialSetup()'s base method doesn't need
// to be called.
void	PortDialog::SetupControls(void)
	{
	// Set some defaults for the buttons
	SetDefaultButton(0);
	SetCancelButton(0);
	
	// Create the controls
	CreateActualControls();
	}
	
	
//@method:PortDialog::ItemHit
//@return:bool
//	True if no more handling is necessary.
//@arg:theItem:PortDialogItem
//	A control id for the item that's been hit.
//@descr
//	Called by DoModal() in response to an user-interaction with a dialog item.
//  Override this in an PortDialog subclass to handle your own control types, and
//  then call the inherited method to handle any built-on ones.  Return "true"
//  if you're done handling, "false" if you want additional system-standard behaviour.
//@endm
bool	PortDialog::ItemHit(PortDialogItem theItem)
	{
	bool returnVal = false;
	long n = iControlList.size();
	if (n != 0)
		{
		// OK, we're a Control-List dialog.  So we're reqponsible for basic handling (they can override
		// this method if they don't like that.)
		sInt16 index = GetControlListIndexFromCommand(theItem);
		
		// Deal with radio groups.
		if ((index != -1) && iControlList[index].iType == kAOControlRadio)
			{
			// Set everything in our group to false, except us.
			for (long i = 0; i < n; i++)
				{
				if (iControlList[i].iType == kAOControlRadio)
					{
					if (iControlList[i].iRadioGroup == iControlList[index].iRadioGroup)
						{
						platformControl ctrl = GetLayoutControl(iControlList[i].iCommandID);
						if (i == index)
							SetPlatformCheckOrRadioValue(ctrl, 1);
						else
							SetPlatformCheckOrRadioValue(ctrl, 0);
						}
					}
				}
			
			}
			
		// And  checkboxes.
		if ((index != -1) && iControlList[index].iType == kAOControlCheckbox)
			{
			platformControl ctrl = GetLayoutControl(iControlList[index].iCommandID);
			if (GetPlatformCheckOrRadioValue(ctrl) == 0)
				SetPlatformCheckOrRadioValue(ctrl, 1);
			else
				SetPlatformCheckOrRadioValue(ctrl, 0);
			}
			
		// They may have special instructions for us.  Don't validate if 
		// cancel, otherwise true.  Might let them specify in XML someday.
		if ((index != -1) && iControlList[index].iCloses)
			CloseDialog(theItem, !iControlList[index].iCancel);
		}
	return returnVal;
	}
	

//@method:PortDialog::CloseDialog
//@arg:result:PortDialogItem
//	The control id to return as the "result" from the DoModal() call.
//@arg:onlyIfValid:bool
//	If true, indicates that the dialog should be closed only if a call to Validate() returns
//  true. (typically this is "true" for OK, "false" for cancel).
//@descr
//	The dialog should end.  The "onlyIfValid" flag means that the user wants
//	validation first, and if the data isn't valid, the "close" should be cancelled.
//	onlyIfValid defaults to TRUE, it should be passes as FALSE for cancel buttons
//	and other circumstances where you don't care if the data's valid.
//@endm
void	PortDialog::CloseDialog(PortDialogItem result, bool onlyIfValid)
	{
	if ((!onlyIfValid) || ValidData())
		{
		iExiting = true;
		iExitValue = result;
		LastChance(result);
		long n = iControlList.size();
		if (n > 0)
			{
			// Store the results of all the controls before we die.
			for (long i = 0; i < n; i++)
				{
				platformControl ctrl = GetLayoutControl(iControlList[i].iCommandID);
				char buffer[256];
				switch (iControlList[i].iType)
					{
					case kAOControlRadio:
					case kAOControlCheckbox:
						{
						iControlList[i].iNumericValue = GetPlatformCheckOrRadioValue(ctrl);
						}
						break;
					case kAOControlEditText:
						GetEditTextText(GetLayoutControl(iControlList[i].iCommandID), iControlList[i].iTextValue);
						break;
					case kAOControlListBox:
						iControlList[i].iNumericValue = SendMessage(ctrl, LB_GETCURSEL, 0, 0);
						SendMessage(ctrl, LB_GETTEXT, iControlList[i].iNumericValue, (long)&buffer[0]);
						iControlList[i].iTextValue = buffer;
						break;
					case kAOControlPulldown:
						iControlList[i].iNumericValue = SendMessage(ctrl, CB_GETCURSEL, 0, 0);
						SendMessage(ctrl, CB_GETLBTEXT, iControlList[i].iNumericValue, (long)&buffer[0]);
						iControlList[i].iTextValue = buffer;
						break;
					default:
						// Presently, no other controls have values.
						break;
					}
				}
			}
		EndDialog(iWindow, result);
		}
	}



//@method:PortDialog::KeyHit
//@return:bool
//  True if no more processing is needed.
//@arg:theChar:char
//  The key they hit.
//@descr
//  Called when a key is hit during DoModal().  Overide if you want first crack at the 
//  keys, but be sure to call the inherited method for standard behaviour on keys you
//  don't handle.
//@endm	

//@method:PortDialog::LastChance
//@arg:exitVal:PortDialogItem
//  The control id of the item that's closing the dialog.
//@descr
//  Called just before a dialog exits, to let you have one last crack at the various
//  controls in it before they're disposed.  This isn't usually necessary with controllist
//  dialogs, since the values of the standard controls are saved for you in the control list.
//  It's still necessary if you have custom controls you need to read.  Override this
//  if you need it; the default handler does nothing.
//@endm
	
//@method:PortDialog::ValidData
//@return:bool
//	true if the data in the dialog is valid
//@descr
//  Called by "CloseDialog()" if the "onlyIfValid" argument is true.  The default
//  handler returns true; you can override this in an PortDialog subclass if you need
//  to do item validation.  Returning "false" from this routine causes the dialog
//  not to close.
//@endm

//@method:PortDialog::IdleTime
//@descr
//  Called intermittently during dialog processing.  The default method does nothing;
//  you can override it in an PortDialog subclass if you need idle time (for a progress
//  bar, for example).
//@endm

} // namespace PortableViews;
#endif
