#ifndef AV_LAYOUT
#define AV_LAYOUT

//@class:Layout
//@descr
// This class provides layout functionality for a set of specified controls.
// Usually, this is subclassed, (PortDialog is a prime example), but it's 
// possible to make a standalone layout object, for laying out a pane of
// a window, for example.

// There are a couple of static methods on Layout for determining
// particular platform metrics (sizes of and distances between standard
// controls), and a single method for performing the actual layout
// (LayoutControls()).  The fundamental structure of this object is the
// control list (iControlList), which is typically filled in by a subclass,
// possibly from an XML <LAYOUT> record.   It is the controls of this
// list which are actually moved around by the LayoutControls() method.
// (There are a couple of accessors for the control list
// SetControlList() and GetControlList(), but they're usually not needed
// because a subclass can set or load the protected iControlList 
// data member directly).

// The header file defines a large number of types and constants for 
// dealing with controls lists, control metrics, and control spacing.
//@endc

#include <vector>
#include "PortString.h"
#include "Controls.h"

namespace PortableViews {

#if (UT_TARGET_MACOS)
	// A mac-only routines that tells us if we're using the
	// Aqua (OS X) appearance or not.
	bool IsAquaAppearance(void);
#endif

// An "command code" in a control list
typedef 	sInt16	LayoutItem;

// The kind of control (pushbutton, checkbox, etc.)
typedef 	controlType LayoutControlType;

// Kinds of inter-item spacing in dialogs (for the GetDialogSpacing() metrics call, below).
typedef 	sInt16 LayoutSpaceType;
	const LayoutSpaceType kAVDSDialogEdge 		= 1;
	const LayoutSpaceType kAVDSGroupBoxEdge 		= 2;
	const LayoutSpaceType kAVDSRelatedControls	= 3;
	const LayoutSpaceType kAVDSUnrelatedControls 	= 4;
	const LayoutSpaceType kAVDSInterButton		= 5;
	const LayoutSpaceType kAVDSInterRadioBox		= 6;
	const LayoutSpaceType kAVDSInterCheckBox		= 7;
	const LayoutSpaceType kAVDSInterEditText		= 8;
	const LayoutSpaceType kAVDSInterIcon			= 9;
	const LayoutSpaceType kAVDSInterPicture		= 10;
	const LayoutSpaceType kAVDSLabelToControl		= 11;
	const LayoutSpaceType kAVDSGroupBoxEdgeBottom = 12;
	

// Information about an individual field in a dialog
struct LayoutControl
	{
	LayoutControlType iType;
	sInt16 				iCommandID;
	PortString			iTitle;
	platformRect		iRect;
	resID				iResourceRef;	// The ICON, BITMAP, MENU, etc. resource used.
	bool				iFixedWidth;
	bool				iFixedHeight;
	sInt16				iRadioGroup;
	bool				iDefault; 		// This control is the default button. (or return/enter)
	bool				iCancel; 		// This control is the cancel button (or esc/Cmd-.)
	bool				iCloses;		// This control closes the dialog (OK/Cancel, usually)
	
	std::vector<PortString>	iListItems;	// Used for items in a list control or pulldown
	
	// These two are used separately or in combination depending on the control type.
	sInt32				iNumericValue;	// Entry: initial state. After DoModal():  final state.
	PortString			iTextValue;	// Entry: initial state. AfterDoModal: final state.
	platformControl		iControl; // Used internally
	};

typedef std::vector<LayoutControl> LayoutControlList;	
	
const sInt16 kLayoutHBox = -1;
const sInt16 kLayoutVBox = -2;
const sInt16 kLayoutFill = -3;
const sInt16 kLayoutSpace = -4;
const sInt16 kLayoutNoTagHere = -5;

// A tree of these things makes up the layout information for a Dialog.  Typically these are
// read from LAYOUT tags in the XML Dialog resources, and you won't have to make your own.
struct LayoutNode {
	sInt16 							control; // Control ID or one of the kLayoutConstants.
	bool							framed;
	PortString						label;
	uInt16							frameIndex; // What Control-list ID is our frame?
	uInt16 							x, y;
	uInt16 							width, height;
	std::vector<LayoutNode> 		children;
};

// Information about the dialog (or window, or other layout) as a whole.
struct PortDialogInfo
	{
	PortString			iTitle;
	sInt16 				iWidth;
	sInt16 				iHeight;
	LayoutNode			iLayoutInfo;
	bool				iLayoutUsed;
	};


// Metrics (size information) about a particular control
struct AVControlMetrics
	{
	bool 	usesLabel;  // Need to include label in width
	uInt16 	height;		// Standard height
	uInt16	minWidth;	// Minimum width
	uInt16 	widthExtra;	// Additional Width added to label width
	uInt16	spacingHeight; // Number of additional units per additional line of text.
	};

class Layout {
protected:
		LayoutControlList	iControlList;
	
protected:
		sInt16	GetControlListIndexFromCommand(sInt16 command);
		
		// Used by the layout manager.
		void	MeasureAllControls(void);
		void	MoveChild(LayoutNode& nd, uInt16 rWidth, uInt16 rHeight);
		void	ExpandFills(LayoutNode&nd, uInt16 tWidth, uInt16 tHeight);
		void	LayoutBox(LayoutNode& nd);
		sInt16	FindControlIndex(LayoutControlList& list, sInt16 command); 
		sInt16	GetSpacingBetweenLayoutNodes(const LayoutNode& n1, 
										 const LayoutNode& n2, bool vertical);

public:
virtual void				LayoutControls(LayoutNode& nd);
		void				SetControlList(const LayoutControlList& newList)
										{ iControlList = newList; }
 		LayoutControlList& 	GetControlList(void) { return iControlList; } 

static	void	GetControlTypeMetrics(LayoutControlType type, AVControlMetrics& metrics);
static	void	GetControlTypeSize(LayoutControlType type, PortString label, 
								uInt16& height, uInt16& width);
static	void GetDialogSpacing(LayoutSpaceType type, sInt16& horiz, sInt16& vert);
};

} // Namespace PortableViews
#endif