// LayoutAll.cpp

// This is the guts of the layout engine.

#include "UniversalSymbols.h"

#include "Controls.h"
#include "Layout.h"
#include "Window.h"

namespace PortableViews {	
//PRIVATE::
// Converts a command-code into an index into the control list.
sInt16	Layout::GetControlListIndexFromCommand(sInt16 command)
	{
	sInt16 returnVal = -1;
	long n = iControlList.size();
	if (n == 0)
		{
		PortThrowAssertException(kUT_RANGE_CHECK, "Command Lookup on non-CommandList dialog.");
		}
	for (long i = 0; i < n; i++)
		{
		if (iControlList[i].iCommandID == command)
			returnVal = i;
		}
	return returnVal;
	}


//PRIVATE:
// As a first step toward laying out the dialog, figure out the height and
// width of each control in the dialog.
void	Layout::MeasureAllControls(void)
	{
	long n = iControlList.size();
	for (long i = 0; i < n; i++)
		{
		LayoutControl& ctrl = iControlList[i];
		uInt16 height = 0, width = 0;
		if (ctrl.iFixedWidth)
			width = ctrl.iRect.right - ctrl.iRect.left;
		Layout::GetControlTypeSize(ctrl.iType, ctrl.iTitle, height, width);
		if (!ctrl.iFixedWidth)
			ctrl.iRect.right = ctrl.iRect.left + width;
		if (!ctrl.iFixedHeight)
			ctrl.iRect.bottom = ctrl.iRect.top + height;
		}
	}


//PRIVATE:
// Moves the node, and all it's children, rWidth and rHeight relative to their
// current position.
void	Layout::MoveChild(LayoutNode& nd, uInt16 rWidth, uInt16 rHeight)
	{
	nd.x += rWidth;
	nd.y += rHeight;
	if (nd.control >= 0)
		{
		sInt16 index = GetControlListIndexFromCommand(nd.control);
		PortAssert(index != -1);
		// We're a control type!
		iControlList[index].iRect.left += rWidth;
		iControlList[index].iRect.top += rHeight;
		iControlList[index].iRect.right += rWidth;
		iControlList[index].iRect.bottom += rHeight;
		}
	else if ((nd.control == kLayoutHBox || nd.control == kLayoutVBox) && nd.framed)
		{
		// Move the frame, too.
		iControlList[nd.frameIndex].iRect.left += rWidth;
		iControlList[nd.frameIndex].iRect.top += rHeight;
		iControlList[nd.frameIndex].iRect.right += rWidth;
		iControlList[nd.frameIndex].iRect.bottom += rHeight;
		}
	long n = nd.children.size();
	if (n > 0)
		for (long i = 0; i < n; i++)
			MoveChild(nd.children[i], rWidth, rHeight);
	}
	
//PRIVATE::
// We've got tWidth width and tHeight height to use up.  If we've got
// space left over, and there are any FILL tags, expand them to use
// up the requisite space.
void	Layout::ExpandFills(LayoutNode& nd,
			uInt16 tWidth, uInt16 tHeight)
	{
	long n = nd.children.size();
	if (n > 0)
		{
		uInt16 extraWidth = tWidth - nd.width;
		uInt16 extraHeight = tHeight - nd.height;
		if ((extraWidth && nd.control == kLayoutHBox) || 
		    (extraHeight && nd.control == kLayoutVBox))
			{
			uInt16 numFills = 0;
			for (long i = 0; i < n; i++)
				if (nd.children[i].control == kLayoutFill)
					numFills++;
					
			if (numFills > 0)
				{
				for (long i = 0; i < n; i++)
					if (nd.children[i].control == kLayoutFill)
						{
						if (nd.control == kLayoutHBox)
							for (long j = i + 1; j < n; j++)
								MoveChild(nd.children[j], (extraWidth/numFills), 0);
						else
							for (long j = i + 1; j < n; j++)
								MoveChild(nd.children[j], 0, extraHeight/numFills);
						}
				}
			}
		// If we're framed, we'll also want to expand the frame to fill the extra
		// space.  This generally looks better.
		if (nd.framed)
			{
			if (extraWidth > 0)
				iControlList[nd.frameIndex].iRect.right += extraWidth;
			if (extraHeight > 0)
				iControlList[nd.frameIndex].iRect.bottom += extraHeight;
			} 
		}
	}
	
//PRIVATE:
// Returns the amound of space between the specified controls in the specified direction.
sInt16	Layout::GetSpacingBetweenLayoutNodes(const LayoutNode& n1, 
											 const LayoutNode& n2, bool vertical)
	{

	// OK, after a bunch of tries, here's the what we'll do for layout:
	
	// The whole Dialog is spaced in from each edge (this is taken care
	// of elsewhere).
	
	// If an HBOX or VBOX is framed, all of it's children are spaced in from
	// it similarly to the controls being spaced in from the edge of the dialog.
	
	// Otherwise, the contents of a box are examined in pairs.  There is no spacing
	// to the "outside" at this level (it will have been applied recursively
	// already):
	//	  If either "side" of the space is a FILL or SPACE, then no spacing is used.
	//	  Else, if either side is a VBOX/HBOX, the "unrelated controls" spacing is used.
	//	  Else, if the controls are the same type, the specific spacing for that
	//			control type is used (this keeps radio button/checkbox/button groups
	//			together, which are typically grouped closer than "related controls"
	//	  Else, the "related controls" spacing is used.

	sInt16 spacing = 0;
	sInt16 relControlSpace, unrelControlSpace, ignore;
	if (vertical)
		{
		GetDialogSpacing(kAVDSRelatedControls, ignore, relControlSpace);
		GetDialogSpacing(kAVDSUnrelatedControls, ignore, unrelControlSpace);
		}
	else
		{
		GetDialogSpacing(kAVDSRelatedControls, relControlSpace, ignore);
		GetDialogSpacing(kAVDSUnrelatedControls, unrelControlSpace, ignore);
		}
	

	sInt16 tControl = n1.control,
	       lControl = n2.control;
	       
	if (	(tControl == kLayoutSpace) ||
	    	(lControl == kLayoutSpace) ||
	    	(tControl == kLayoutFill)  ||
	   		(lControl == kLayoutFill))
	    spacing = 0;
	else if ((tControl == kLayoutHBox) ||
			 (lControl == kLayoutHBox) ||
			 (tControl == kLayoutVBox) ||
			 (lControl == kLayoutVBox))
		spacing = unrelControlSpace;
	else
		{
		sInt16 index1 = GetControlListIndexFromCommand(tControl);
		sInt16 index2 = GetControlListIndexFromCommand(lControl);
		PortAssert(index1 != -1);
		PortAssert(index2 != -1);
		LayoutControlType type1 = iControlList[index1].iType;
		LayoutControlType type2 = iControlList[index2].iType;	       

		if (type1 == type2)
			{
		sInt16 h, v;
			// Same type of control, use special spacing.
			switch (type1)
				{
				case kAOControlButton:
					GetDialogSpacing(kAVDSInterButton, h, v);
					spacing = (vertical) ? v : h;
					break;
				case kAOControlRadio:
					GetDialogSpacing(kAVDSInterRadioBox, h, v);
					spacing = (vertical) ? v : h;
					break;
				case kAOControlCheckbox:
					GetDialogSpacing(kAVDSInterCheckBox, h, v);
					spacing = (vertical) ? v : h;
					break;
				case kAOControlEditText:
					GetDialogSpacing(kAVDSInterEditText, h, v);
					spacing = (vertical) ? v : h;
					break;
				default:
					spacing = relControlSpace;
				}
			}
		else
			spacing = relControlSpace;			
		}
		
	return spacing;
	}
	
//PRIVATE:
// Lays out the dialog box using the layout information in the LayoutInfo.
void	Layout::LayoutBox(LayoutNode& nd)
	{
	long n = nd.children.size();
	
	if (n > 0)
		{
		// We are an HBOX or VBOX.  Recursivly layout our children.
		PortAssert(nd.control == kLayoutHBox  || nd.control == kLayoutVBox);
		for (long i = 0; i < n; i++)
			LayoutBox(nd.children[i]);
			
		uInt16 height = nd.children[0].height, width = nd.children[0].width;
		sInt16 spacing = 0;
		for (long i = 1; i < n; i++)
			{
			if (nd.control == kLayoutHBox)
				{
				width += nd.children[i].width;
				height = std::max(nd.children[i].height, height);
				spacing += GetSpacingBetweenLayoutNodes(
								nd.children[i], nd.children[i-1], false);
				}
			if (nd.control == kLayoutVBox)
				{
				height += nd.children[i].height;
				width = std::max(nd.children[i].width, width);
				spacing += GetSpacingBetweenLayoutNodes(
								nd.children[i], nd.children[i-1], true);
				}
			}
			
		nd.height = height + ((nd.control == kLayoutHBox) ? 0 : spacing);
		nd.width = width + ((nd.control == kLayoutHBox) ? spacing : 0);
		
		sInt16 gbh, gbv;
		sInt16 gbhbot, gbvbot;
		GetDialogSpacing(kAVDSGroupBoxEdge, gbh, gbv);
		GetDialogSpacing(kAVDSGroupBoxEdgeBottom, gbhbot, gbvbot);
		if (nd.framed)
			{
			// We're framed, so we need to:
			// a) add some space to our width and height for the frame itself
			//    and the frame margin, and
			nd.height += gbv + gbvbot;
			nd.width += gbh + gbhbot;
			 
			// b) create a new frame control and add it to the end of control list.
			LayoutControl ctrl;
			ctrl.iType = kAOControlFramedBox;
			// Since we might have to manipulate it later, we have to give it a control ID.
			// Start them at 20000, just increment;
			static int genCommandID = 20000;
			ctrl.iCommandID = genCommandID++;
			ctrl.iTitle = nd.label;
			SetRectTLBR(&ctrl.iRect, 0, 0, nd.height, nd.width); 
			ctrl.iFixedWidth = true;
			ctrl.iFixedHeight = true;
			nd.frameIndex = iControlList.size();
			iControlList.push_back(ctrl);
			}
			
		// We know our own width and height now, let the children have the breathing
		// room. (Note, this doesn't include the spacing, frame, or frame margins. That's extra "hard"
		// space that the fills can't have).  Also, we only allow them to expand perpendicular to
		// the direction of flow (A fill in an HBOX in an HBOX shouldn't generate any space, for example,
		// that space belongs to the parent control.)
		for (long i = 0; i < n; i++)
			if (nd.control == kLayoutHBox)
				ExpandFills(nd.children[i], nd.children[i].width, height);
			else
				ExpandFills(nd.children[i], width, nd.children[i].height);
			
		// Finally, move all of our children to their relative positions.
		// Except the first, which is in the right place.
		uInt16 cWidth = nd.children[0].width;
		uInt16 cHeight = nd.children[0].height;
		if (nd.framed && nd.control == kLayoutHBox)
			{
			MoveChild(nd.children[0], gbh, gbv);
			cWidth += gbh;
			}
		else if (nd.framed && nd.control == kLayoutVBox)
			{
			MoveChild(nd.children[0], gbh, gbv);
			cHeight += gbv;
			}
		for (long i = 1; i < n; i++)
			{
			if (nd.control == kLayoutHBox)
				{
				spacing = GetSpacingBetweenLayoutNodes(
								nd.children[i], nd.children[i-1], false);
				MoveChild(nd.children[i], cWidth + spacing, (nd.framed) ? gbv : 0);
				cWidth += nd.children[i].width + spacing;
				}
			if (nd.control == kLayoutVBox)
				{
				spacing = GetSpacingBetweenLayoutNodes(
								nd.children[i], nd.children[i-1], true);
				MoveChild(nd.children[i], (nd.framed) ? gbh : 0, cHeight + spacing);
				cHeight += nd.children[i].height + spacing;
				}
			}
		}
	else
		{
		// OK, we're a FILL, SPACE, or control.
		switch (nd.control)
			{
			case kLayoutFill:
				// Our size is zero, for the moment.  We'll get expanded later,
				// if possible.
				nd.width = 0;
				nd.height = 0;
				break;
			case kLayoutSpace:
				// Height and width have already been assigned.
				break;
			default:
				// OK, we're a control.
				sInt16 index = GetControlListIndexFromCommand(nd.control);
				PortAssert(index != -1);
				nd.height = iControlList[index].iRect.bottom - 
						    iControlList[index].iRect.top;
				nd.width =  iControlList[index].iRect.right - 
						    iControlList[index].iRect.left;
				nd.x = 0;
				nd.y = 0;
#if (UT_TARGET_WIN32)
				// Static text controls need a small vertical bump on Windows to make them
				// line up with in-line radio buttons.
				if (iControlList[index].iType == kAOControlStaticText)
					nd.y += 1;
#endif
				iControlList[index].iRect.left = nd.x;
				iControlList[index].iRect.top = nd.y;
				iControlList[index].iRect.right = nd.x + nd.width;
				iControlList[index].iRect.bottom = nd.y + nd.height;
				break;
			}
		}
	}



//@method:Layout::LayoutControls
//@arg:topNode:LayoutNode reference
// The top of a layout node tree.
//@descr
//  This method actually causes the controls in iControlList
//  to be arranged according to the specified tree of layout nodes.  You'll rarely ever have to call
//  this directly, usually it's done on your behalf by a subclass such as PortDialog.  On exit, the
//  control rectangles in iControlList have been altered to reflect the specified layout.
//  Note that this function may increase the number of controls in iControlList (it will add frames
//  and actual controls for things that can't be specified as dialog items), so the size of the list
// 	should not be assumed to be a constant over a call to this function.
//@endm
void	Layout::LayoutControls(LayoutNode& nd)
	{
	MeasureAllControls();
	LayoutBox(nd);
	}

} // Namespace PortableViews