// LayoutMac.cpp

// Mac OS implementation of a couple of Layout methods.
#include "UniversalSymbols.h"
#if (UT_TARGET_MACOS)

#include "Controls.h"
#include "Layout.h"
#include "Window.h"
#include <Gestalt.h>
#include <Appearance.h>

namespace PortableViews {	

// Returns "True" if we're using Aqua, false if we're using Platinum appearance.
// The major difference is in control layout; Platinum uses a fairly dense arrangement
// similar to windows, Aqua tries to sell you a 36" monitor.
bool	IsAquaAppearance(void)
	{
	long result;
	Gestalt(gestaltMenuMgrAttr, &result);
	return (result & gestaltMenuMgrAquaLayoutMask) > 0;
	}

namespace {
		
//@method:AVDC::GetTextLineBreaks
//@arg:text:const UtString reference
//  The text that you wish to break
//@arg:width:sInt16
//  The maximum width for a line, in pixels
//@arg:breaks:std::vector<int> reference
//	On exit, the index of the first character of each line.
//@descr
//  This method measures the specified text in the current point size,
//  determines how much text can fit on each line, and returns in
//  "breaks" the index of the character that begins each line.  Implicitly,
//  breaks.size() is the number of lines.
//
//  The rules: 
//
//  No line is longer than "width" (unless "width" is too small to allow a
//  single character). 
//
// 	Lines are broken on spaces, unless a run of characters longer than "width" occurs
//  without any spaces. 
//
//	Spaces are trimmed from the front and end of lines.
//@endm
void	GetTextLineBreaks(const PortString& text, sInt16 width, std::vector<uInt16>& breaks)
	{
	const char* 	textPtr = (const char*)text;
	int 	len = text.Length();
	
	if (len == 0)
		{
		breaks.push_back(0);
		}
	else
		{
		short pos[257];
		
		// Get the offsets
		MeasureText(text.Length(), (const char *)text, &pos);
			
		int linebegin = 0;
		int remaining = pos[len];
		do
			{
			while (linebegin == ' ' && linebegin < len)
				linebegin++;
			if (linebegin < len)
				{
				remaining = pos[len] - pos[linebegin]; 
				breaks.push_back(linebegin);

				int charpos = linebegin;
				while ((charpos < len) && (pos[charpos] - pos[linebegin] < width))
					charpos++;
				
				// At this point, charpos points to the "overflowing" character.
				if (charpos == len)
					{
					// Don't have to do anything, it all fits.
					}
				else if (text[charpos] == ' ')
					// If it's a space, we're fine.
					linebegin = charpos;
					
				else if (charpos <= linebegin+1)
					// We don't have room for even a single character.
					linebegin = linebegin+1;
				
				else
					{
					int lineend = charpos-1;
					// Backup to the last space, but not before the beginning of the line.
					while (lineend > linebegin && text[lineend] != ' ')
						lineend--;
					if (lineend == linebegin)
						// No spaces, just have to live with a break in the middle of a word
						linebegin = charpos;
					else
						// Start the next line just after the space.
						linebegin = lineend+1;
					}
				}
			} while (remaining > width);
		}
	}

}
	
//@function:Layout::GetControlTypeSize [static]
//@arg:type:controlType
// The type of the control (see constants in Layout.h)
//@arg:label:PortString
// The label to be attached to the control
//@arg:height:uInt16 reference
// On return, the height of the control, in pixels (mac) or dialog units(win).
//@arg:width:uInt16 reference
// On return, the width of the control, in pixels (mac) or dialog units(win).
//@descr
// [Static] Fills in the height and width of a control of the specified type, with the
// specified label, for the given platform and runtime environment.  Used
// internally by the dialog layout manager, and useful for dynamic control environments.
//@endf
void	Layout::GetControlTypeSize(LayoutControlType type, PortString label, 
						uInt16& height, uInt16& width)
	{
	AVControlMetrics m;
	GetControlTypeMetrics(type, m);
	
	uInt16 fixedWidth = width;
	
	height = m.height;
	width = m.widthExtra;
	if (m.usesLabel)
		{
		// Need a temporary DC in order to do measurements.	
		// Make a 1x1x32 GWorld temporarily.  If you have another
		// port convenient, or if you know the current port is valid,
		// this rather expensive step can be eliminated.	
		GrafPtr port;
		GetPort(&port);
		platformRect frame;
		SetRectTLBR(&frame, 0, 0, 1, 1);
		GWorldPtr tmpWorld;
		OSErr err = NewGWorld(&tmpWorld, 16, &frame, NULL, NULL, keepLocal);
		SetGWorld(tmpWorld, 0);
		UseThemeFont(kThemeSystemFont, smSystemScript);
		
		if (fixedWidth != 0)
			{
			std::vector<uInt16> breaks;
			platformFontMetrics fm;
			GetTextLineBreaks(label, fixedWidth, breaks);
			::GetFontInfo(&fm);
			sInt16 lineHeight = std::max((uInt16)(GetMetricAscent(fm) + GetMetricDescent(fm) + 2),
										 m.spacingHeight);
			height = m.height +  
				((breaks.size() - 1) * lineHeight);
			width = fixedWidth;
			}
		else
			{
			width += StringWidth((PStringPtr)label);
			width += 1;
			}
		SetPort(port);
		DisposeGWorld(tmpWorld);
		}
	
	if (width < m.minWidth)
		width = m.minWidth;
	}
	
	
//@function:Layout::GetControlTypeMetrics [static]
//@arg:type:controlType
// The type of the control (see constants in Layout.h)
//@arg:metrics:AVControlMetrics reference
// Metrics record to be filled in.
//@descr
// [Static] Fills in an AOControlMetrics record for the specified type of control,
// based on the current platform and runtime environment (which is a 
// fancy way of saying it knows about Aqua on the Mac). Used
// internally by the dialog layout manager, and useful for dynamic control environments.
//
// On windows, I've adjusted these to what Windows programs actually use, instead of what
// the user interface guidelines SAY they use.  Your buttons should be better behaved now.
//@endf
void	Layout::GetControlTypeMetrics(LayoutControlType type, AVControlMetrics& metrics)
	{
	if (IsAquaAppearance())
		{
		switch (type)
			{
			case kAOControlButton:
				metrics.usesLabel = true;
				metrics.height = 20;
				metrics.spacingHeight = 0;
				metrics.minWidth = 69;
				metrics.widthExtra = 28;
				break;
			case kAOControlCheckbox:
			case kAOControlRadio:
				metrics.usesLabel = true;
				metrics.height = 18;
				metrics.spacingHeight = 14;
				metrics.minWidth = 80;
				metrics.widthExtra = 26;
				break;
			case kAOControlStaticText:
				metrics.usesLabel = true;
				metrics.height = 16;
				metrics.spacingHeight = 14;
				metrics.minWidth = 0;
				metrics.widthExtra = 2;
				break;
			case kAOControlEditText:
				metrics.usesLabel = true;
				metrics.height = 22;
				metrics.spacingHeight = 14;
				metrics.minWidth = 100;
				metrics.widthExtra = 0;
				break;
			case kAOControlPulldown:
				metrics.usesLabel = false;
				metrics.height = 40;
				metrics.minWidth = 69;
				metrics.widthExtra = 38;
				break;
			case kAOControlListBox:
				metrics.usesLabel = false;
				metrics.height = 20;
				metrics.minWidth = 69;
				metrics.widthExtra = 38;
				break;
			}
		}
	else
		{
		switch (type)
			{
			case kAOControlButton:
				metrics.usesLabel = true;
				metrics.height = 20;
				metrics.spacingHeight = 12;
				metrics.minWidth = 58;
				metrics.widthExtra = 16;
				break;
			case kAOControlIcon: // For icon/picture, hope they supplied dimensions...
				metrics.usesLabel = false;
				metrics.height = 32;
				metrics.spacingHeight = 12;
				metrics.minWidth = 32;
				metrics.widthExtra = 0;
				break;
			case kAOControlPicture:
				metrics.usesLabel = false;
				metrics.height = 32;
				metrics.minWidth = 32;
				metrics.widthExtra = 0;
				break;
			case kAOControlCheckbox:
			case kAOControlRadio:
				metrics.usesLabel = true;
				metrics.height = 18;
				metrics.spacingHeight = 12;
				metrics.minWidth = 80;
				metrics.widthExtra = 26;
				break;
			case kAOControlStaticText:
				metrics.usesLabel = true;
				metrics.height = 14;
				metrics.spacingHeight = 16;
				metrics.minWidth = 0;
				metrics.widthExtra = 2;
				break;
			case kAOControlEditText:
				metrics.usesLabel = true;
				metrics.height = 20;
				metrics.spacingHeight = 12;
				metrics.minWidth = 100;
				metrics.widthExtra = 0;
				break;
			case kAOControlPulldown:
				metrics.usesLabel = false;
				metrics.height = 20;
				metrics.spacingHeight = 12;
				metrics.minWidth = 58;
				metrics.widthExtra = 20;
				break;
			case kAOControlListBox:
				metrics.usesLabel = false;
				metrics.height = 20;
				metrics.spacingHeight = 12;
				metrics.minWidth = 58;
				metrics.widthExtra = 20;
				break;
			}
		}
	}


//@function:Layout::GetDialogSpacing [static]
//@arg:type:LayoutSpaceType
// The type of the space you want the size of (see Layout.h for constants).
//@arg:horiz:sInt16 reference
// On return, the horizontal spacing, in pixels.
//@arg:vert:sInt16 reference
// On return, the vertical spacing, in pixels.
//@descr
// Returns the number of pixels that should be used in the 
// specified context.  Used internally by the XML Dialog layout manager.
// On the Mac, knows whether you're in aqua or not, and deals accordingly.
//
// On windows, I've adjusted these to what Windows programs actually use, instead of what
// the user interface guidelines SAY they use.  Your buttons should be better behaved now.
//@endf
void	Layout::GetDialogSpacing(LayoutSpaceType type, sInt16& horiz, sInt16& vert)
	{
	if (IsAquaAppearance())
		{
		switch (type)
			{
			case kAVDSDialogEdge:
				horiz = 13;
				vert = 13;
				break;
			case kAVDSGroupBoxEdge:
				horiz = 16;
				vert = 20;
				break;
			case kAVDSGroupBoxEdgeBottom:
				horiz = 16;
				vert = 10;
				break;
			case kAVDSRelatedControls:
				horiz = 12;
				vert = 16;
				break;
			case kAVDSUnrelatedControls:
				horiz = 20;
				vert = 20;
				break;
			case kAVDSInterButton:
				horiz = 12;
				vert = 12;
				break;
			case kAVDSInterRadioBox:
				horiz = 3;
				vert = 4;
				break;
			case kAVDSInterCheckBox:
				horiz = 3;
				vert = 4;
				break;
			case kAVDSInterIcon:
				horiz = 4;
				vert = 4;
				break;
			case kAVDSInterPicture:
				horiz = 4;
				vert = 4;
				break;
			case kAVDSInterEditText:
				horiz = 8;
				vert = 8;
				break;
			case kAVDSLabelToControl:
				horiz = 8;
				vert = 6;
				break;
			}
		}
	else
		{
		switch (type)
			{
			case kAVDSDialogEdge:
				horiz = 12;
				vert = 12;
				break;
			case kAVDSGroupBoxEdge:
				horiz = 10;
				vert = 20;
				break;
			case kAVDSRelatedControls:
				horiz = 12;
				vert = 8;
				break;
			case kAVDSUnrelatedControls:
				horiz = 16;
				vert = 12;
				break;
			case kAVDSInterButton:
				horiz = 12;
				vert = 10;
				break;
			case kAVDSInterRadioBox:
				horiz = 3;
				vert = 0;
				break;
			case kAVDSInterCheckBox:
				horiz = 3;
				vert = 0;
				break;
			case kAVDSInterIcon:
				horiz = 4;
				vert = 4;
				break;
			case kAVDSInterPicture:
				horiz = 4;
				vert = 4;
				break;
			case kAVDSInterEditText:
				horiz = 6;
				vert = 6;
				break;
			case kAVDSLabelToControl:
				horiz = 3;
				vert = 6;
				break;
			case kAVDSGroupBoxEdgeBottom:
				horiz = 10;
				vert = 10;
				break;
			}
		}

	}

} // Namespace PortableViews
#endif //UT_TARGET_MACOS
