// LayoutWin.cpp

// Windows implementation of a couple of Layout methods.

#include "UniversalSymbols.h"

#if (UT_TARGET_WIN32)

#include "Controls.h"
#include "Layout.h"
#include "Window.h"

namespace PortableViews {	

namespace {
void	GetTextLineBreaks(HDC dc, const PortString& text, sInt16 width, std::vector<uInt16>& breaks)
	{
	const char* 	textPtr = (const char*)text;
	int 	len = text.Length();
	
	if (len == 0)
		{
		breaks.push_back(0);
		}
	else
		{
		SIZE theSize;
		int pos[256];
		
		// Get the offsets
		GetTextExtentExPoint(dc, (const char *)text, text.Length(), 0, NULL, 
			pos, &theSize);
			
		int linebegin = 0;
		int remaining = theSize.cx;
		do
			{
			while (linebegin == ' ' && linebegin < len)
				linebegin++;
			if (linebegin < len)
				{
				remaining = theSize.cx - pos[linebegin]; 
				breaks.push_back(linebegin);

				int charpos = linebegin;
				while ((charpos < len) && (pos[charpos] - pos[linebegin] < width))
					charpos++;
				
				// At this point, charpos points to the "overflowing" character.
				if (charpos == len)
					{
					// Don't have to do anything, it all fits.
					}
				else if (text[charpos] == ' ')
					// If it's a space, we're fine.
					linebegin = charpos+1;
					
				else if (charpos <= linebegin+1)
					// We don't have room for even a single character.
					linebegin = linebegin+1;
				
				else
					{
					int lineend = charpos-1;
					// Backup to the last space, but not before the beginning of the line.
					while (lineend > linebegin && text[lineend] != ' ')
						lineend--;
					if (lineend == linebegin)
						// No spaces, just have to live with a break in the middle of a word
						linebegin = charpos;
					else
						// Start the next line just after the space.
						linebegin = lineend+1;
					}
				}
			} while (remaining > width);
		}
	}
}


//@function:Layout::GetControlTypeSize [static]
//@arg:type:controlType
// The type of the control (see constants in Layout.h)
//@arg:label:PortString
// The label to be attached to the control
//@arg:height:uInt16 reference
// On return, the height of the control, in pixels (mac) or dialog units(win).
//@arg:width:uInt16 reference
// On return, the width of the control, in pixels (mac) or dialog units(win).
//@descr
// [Static] Fills in the height and width of a control of the specified type, with the
// specified label, for the given platform and runtime environment.  Used
// internally by the dialog layout manager, and useful for dynamic control environments.
//@endf
void	Layout::GetControlTypeSize(LayoutControlType type, PortString label, 
						uInt16& height, uInt16& width)
	{
	AVControlMetrics m;
	GetControlTypeMetrics(type, m);
	
	uInt16 fixedWidth = width;
	
	height = m.height;
	width = m.widthExtra;
	if (m.usesLabel)
		{
		// Need a temporary DC in order to do measurements.
		HDC theDC = GetDC(NULL);
		
		SelectObject(theDC, GetStockObject(DEFAULT_GUI_FONT));
		
		if (fixedWidth != 0)
			{
			std::vector<uInt16> breaks;
			platformFontMetrics fm;
			GetTextLineBreaks(theDC, label, fixedWidth, breaks);
			::GetTextMetrics(theDC, &fm);
			height = m.height +  
				((breaks.size() - 1) * (GetMetricAscent(fm) + GetMetricDescent(fm)));
			width = fixedWidth;
			}
		else
			{
			SIZE theSize;
			::GetTextExtentPoint32(theDC, (const char *)label, label.Length(), &theSize);
			width += theSize.cx;
			width += 1;
			}
			
		ReleaseDC(NULL, theDC);
		}
	
	if (width < m.minWidth)
		width = m.minWidth;
	}
	
	

void	Layout::GetControlTypeMetrics(LayoutControlType type, AVControlMetrics& metrics)
	{
	switch (type)
		{
		case kAOControlButton:
			metrics.usesLabel = true;
			metrics.height = 22;
			metrics.spacingHeight = 0;
			metrics.minWidth = 75;
			metrics.widthExtra = 10;
			break;
		case kAOControlIcon: // For icon/picture, hope they supplied dimensions...
			metrics.usesLabel = false;
			metrics.height = 32;
			metrics.spacingHeight = 0;
			metrics.minWidth = 32;
			metrics.widthExtra = 0;
			break;
		case kAOControlPicture:
			metrics.usesLabel = false;
			metrics.height = 32;
			metrics.spacingHeight = 0;
			metrics.minWidth = 32;
			metrics.widthExtra = 0;
			break;
		case kAOControlCheckbox:
		case kAOControlRadio:
			metrics.usesLabel = true;
			metrics.height = 20;
			metrics.spacingHeight = 12;
			metrics.minWidth = 40;
			metrics.widthExtra = 36;
			break;
		case kAOControlStaticText:
			metrics.usesLabel = true;
			metrics.height = 16;
			metrics.spacingHeight = 8;
			metrics.minWidth = 0;
			metrics.widthExtra = 1;
			break;
		case kAOControlEditText:
			metrics.usesLabel = true;
			metrics.height = 20;
			metrics.spacingHeight = 8;
			metrics.minWidth = 100;
			metrics.widthExtra = 0;
			break;
		case kAOControlPulldown:
			metrics.usesLabel = true;
			metrics.height = 20;
			metrics.spacingHeight = 8;
			metrics.minWidth = 60;
			metrics.widthExtra = 20;
			break;
		case kAOControlListBox:
			metrics.usesLabel = false;
			metrics.height = 50;
			metrics.spacingHeight = 8;
			metrics.minWidth = 60;
			metrics.widthExtra = 0;
			break;
		default:
			PortAssert(false);
			break;
		}
	}

	
//STATIC::	
void	Layout::GetDialogSpacing(LayoutSpaceType type, sInt16& horiz, sInt16& vert)
	{
	switch (type)
		{
		case kAVDSDialogEdge:
			horiz = 14;
			vert = 14;
			break;
		case kAVDSGroupBoxEdge:
			horiz = 8;
			vert = 22;
			break;
		case kAVDSGroupBoxEdgeBottom:
			horiz = 20;
			vert = 10;
			break;
		case kAVDSRelatedControls:
			horiz = 8;
			vert = 8;
			break;
		case kAVDSUnrelatedControls:
			horiz = 14;
			vert = 14;
			break;
		case kAVDSInterButton:
			horiz = 4;
			vert = 4;
			break;
		case kAVDSInterRadioBox:
			horiz = 6;
			vert = 0;
			break;
		case kAVDSInterCheckBox:
			horiz = 6;
			vert = 0;
			break;
		case kAVDSInterIcon:
			horiz = 4;
			vert = 4;
			break;
		case kAVDSInterPicture:
			horiz = 4;
			vert = 4;
			break;
		case kAVDSInterEditText:
			horiz = 8;
			vert = 4;
			break;
		case kAVDSLabelToControl:
			horiz = 4;
			vert = 4;
			break;
		}
	}

} // Namespace PortableViews
#endif // UT_TARGET_WIN32
