//
//	PortErrors.h	(Not Portable)

//@collection:PortErrors
//@descr
//	This file defines error-handling methods, classes, and macros.  It is included, directly or
//	otherwise, by almost everything.
//
//	A brief reminder for those new at this stuff:
//
//	Asserts (In the form of PortAssert) are used for *programming* errors; i.e. catching
//	situations that are NEVER supposed to arise during the running if your program.  The
//	argument to PortAssert is not evaluated AT ALL (so don't put function calls in there!)
//	when UT_DEBUG is false, so they offer no protection in a final product.
//
//	Exceptions are for *run-time* errors and other exceptional conditions; they are designed
//	to let you catch and recover from rare but possible error conditions that arise during
//	the running of a program (disk full, out of memory, etc).  They offer a few syntactic
//	and structural advantages over return values, including easy deferral of the error to
//	whatever level of the program can handle it, but involve some programming techniques that
//	aren't immediately obvious.  A full description is WAY beyond the scope of this document.
//	Try the "Effective C++" CD-ROM by Scott Meyers as a starting place.

//  What if you need both?  There's a PortThrowAssertException() macro that Asserts (if debugging), then throws
//  an exception.  You'll see it used a lot in the sample code, since there are lots of grey 
//  areas between the two above cases; and you'll often want to know during debugging when an exception
//  fires.  Alternatively, most debugging environments have a "Break on C++ Exceptions" mechanism,
//  which is a source-independent way of doing the same thing.  Use whichever you prefer.
//@endc

#ifndef 	UT_ERROR
#define		UT_ERROR

namespace PortableViews {

typedef	sInt32				errorResult;

// Errors are constants.  We don't want to have to define them all here, so
// we'll reserve the high byte as a facility code, and just declare the facilities.
// (And even that's just to prevent overlap).  Facility codes 0-128 are for user applications.
// the rest (129-255) are reserved to PortableViews.  0 and -1 are special cases, so they're
// unavailable for user allocation.

// There are two universal errors: kUT_NO_ERROR (successful completion), and kUT_ERROR_RESULT,
// which is a generic "something went wrong" code.
const 	errorResult	kUT_NO_ERROR		= 0;
const	errorResult	kUT_ERROR_RESULT	= -1;

// Then we declare the facilities.
const	errorResult	kUTFAC_OS			= 0xFF000000;
const 	errorResult kUTFAC_STREAM		= 0xFE000000;
const	errorResult kUTFAC_CONTAINERS	= 0xFD000000;
const	errorResult	kUTFAC_NETWORKING	= 0xFC000000;
const 	errorResult kUTFAC_HIERARCHY	= 0xFB000000;
const	errorResult	kUTFAC_FILESYSTEM	= 0xFA000000;
const	errorResult	kUTFAC_VIEWS		= 0xF9000000;
const 	errorResult kUTFAC_DATATRANSFER = 0xF8000000;

// Here's a constant for you to use, if you don't want to declare your own for all 
// different error "categories".
const 	errorResult kUTFAC_USERERRORS	= 0x7F000000;

// And some error codes using them. These are chosen because they're generic enough to be used in
// a lot of places.  More specific ones are found in the individual systems .h files.

// Container codes:
const	errorResult kUT_RANGE_CHECK 	= kUTFAC_CONTAINERS + 1; // Out of bounds array access
const	errorResult	kUT_SEARCH_FAILED	= kUTFAC_CONTAINERS + 2; // No such item
const	errorResult kUT_NO_MORE_ITEMS	= kUTFAC_CONTAINERS + 3; // End of list reached
const 	errorResult kUT_WRONG_TYPE		= kUTFAC_CONTAINERS + 4; // Key is of wrong type
const 	errorResult kUT_INVALID_STRUCTURE = kUTFAC_CONTAINERS + 5; // The container's structure is corrupt
const 	errorResult kUT_NO_DUPLICATE	= kUTFAC_CONTAINERS + 6; // This container does not allow duplicates.

// OS Codes:
const	errorResult	kUT_OUT_OF_MEMORY	= kUTFAC_OS + 1; // Memory allocation failed
const 	errorResult kUT_RES_NOT_FOUND	= kUTFAC_OS + 2; // Requested resource does not exist
const 	errorResult kUT_USER_CANCELLED 	= kUTFAC_OS + 3; // User cancelled an operation
const	errorResult kUT_ENT_NOT_CREATED = kUTFAC_OS + 4; // Attempt to create some entity failed.

//
//@class:UtException
//@descr
//		An exception class for PortableViews.  Use one of these by creating it with
//		and error code and an optional string.  The default "catch" block will just
//		print out the error code, error code map string, if any, and number.
//@endc
class 	UtException {
protected:
	char		iText[80];
	char		iFile[80];
	uInt32		iLine;
	errorResult	iError;
public:
				UtException(errorResult err, char* text, char* file, uInt32 line);
	void		Display(void);
	errorResult	GetErrorType(void) { return iError; }
};

//
//@function:PortErrors::PortAssert
//@descr
//			PortAssert is a macro which (assuming the symbol UT_DEBUG is true) will
//			call the PortAssertFailed function.  Although I usually prefer to avoid
//			macros, this one is special -- we want it to become zero-overhead if
//			UT_DEBUG is false (without relying on superhuman compiler optimization),
//			and we want to expand the file and line number macros for precise error
//			locating.  "Expression" is only evaluated once during the expansion, so
//			most irritating problems should not occur.
//
//	Note:	if UT_DEBUG is false, "expression" is never evaluated!
//@endf
void PortAssertFailed (constCStringPtr theFile, sInt32 theLine, constCStringPtr expression);

#if (UT_DEBUG)
	#define PortAssert(expression)	{if (!(expression)) PortAssertFailed (__FILE__, __LINE__, #expression); }
#else
	#define PortAssert(expression)
#endif

//@function:PortErrors::PortThrowException
//@descr
// A Macro to throw a particular exception.
//@endf
#define	PortThrowException(error, text) { throw UtException(error, text, __FILE__, __LINE__); }

//@function:PortErrors::PortThrowAssertException
//@descr
// Same thing as PortThrowException, but assert first, so you can debug before your context is lost.
//@endf
#define PortThrowAssertException(error, text) { PortAssert(false); throw UtException(error, text, __FILE__, __LINE__); }


} // namespace PortableViews

#endif