//
//	PortString.h

// A portable string class that knows how to deal with both
// C strings and Mac-centric Pascal Strings.  Note that these don't deal
// with Unicode, so it's not a very good general string class; it's just
// enough to make the provided code work.

#ifndef 	UT_STRING
#define		UT_STRING

#include <string>
#include <cstring>

namespace PortableViews {
class UtSymbolic;
const unsigned char kNullChar = '\0';

void	GetPStringFromResource(PStringPtr theString, const resID stringNum);
void	GetPStringFromResourceTable(PStringPtr theString, const uInt8 table, const uInt8 stringNum);

//@class:PortString
//@descr

// Special, today only!  a handy-dandy string class to encapsulate some of this stuff!
// Generates a 255-character-max string that functions as a C or Pascal String
// This string always allocates 257 bytes (length + 255 + null), so you can write
// to it (within the 255 character bounds), so long as you remember to resolve afterward.
// Note: This _depends_ on unsigned char *'s being pascal strings, and char *'s as
// being C strings.

// There are all sorts of handy routines defined here, and they'll auto-convert as 
// necessary to whatever type of string you <strong>meant</strong> to pass, but there's
// a caveat -- if you write to one, you'll need to call one of the "ResolveAfterxxxWrite" 
// routines to keep it in synch.   Luckily, you don't need to write to strings very often.
// In my own routines I tend to build the strings in native C-string format, then assign them
// to an PortString when I'm done.
//@endc


class PortString {
protected:
	unsigned char iString[257];
		
public:
//@method:PortString::PortString
//@arg:initialvalue:void, constCStringPtr, constPStringPtr, or std::string
//@descr
// Created the string, either empty, or with a copy of the specified string.
//@endm
	PortString(void); // Generate an empty string;
	PortString(constPStringPtr theString) { SetValue(theString); } // Generate from a Pascal String;
	PortString(constCStringPtr theString) { SetValue(theString); } // Generate from a C string
	PortString(const std::string& theString) { SetValue(theString); }
	
//@method:PortString::SetEmpty
//@descr
//  Erase the string.
//@endm
	void SetEmpty(void) { iString[0] = 0; iString[1] = 0; }
	
//@method:PortString::SetValue
//@arg:newValue:constCStringPtr, constPStringPtr, std::string, sInt32, or PortString
//@descr
// Replace the string's value with the specified new one.  For the sInt32 case, the integer
// is automatically converted to a string before appending.
//@endm
	void SetValue(constPStringPtr theString); // Throw away existing, replace with theString
	void SetValue(constCStringPtr theString);
	void SetValue(const std::string& theString);
	void SetValue(sInt32 theNum);
	void SetValue(const PortString& theString) { *this = (constCStringPtr)theString; }
	
//@method:PortString::SetValueFromResource
//@arg:strRes:resID
// The string resource to load the string from
//@descr
// Replace the string's value with the specified new one from a resource.  An alternate
// form takes a uInt8 table ID, and a uInt8 index.
//@endm
	void SetValueFromResource(resID strRes);
	void SetValueFromResource(uInt8 strnRes, uInt8 index);
	
//@method:PortString::GetStdString
//@return:standard string
// A standard string (std::string) containing the PortString's value.
//@descr
// Returns the current string value as a C++ standard library string.
//@endm
	std::string GetStdString(void);
	
//@method:PortString::Append
//@arg:newValue:constCStringPtr, constPStringPtr, std::string, sInt32, or PortString
//@descr
// Appends the new value to the current value.  For the sInt32 case, the integer
// is automatically converted to a string before appending.
//@endm
	void Append(constPStringPtr appendThis);
	void Append(constCStringPtr appendThis);
	void Append(sInt32 appendThis);
	void Append(const PortString& theString) { Append((const char*)theString); }
	void Append(const std::string& theString) {Append(theString.c_str());}
	
//@method:PortString::Left
//@arg:number:uInt8
// How many character in the desired result string
//@arg:result:PortString reference
//@descr
// Returns the first "number" characters of the string as a result string.
//@endm
	void Left(uInt8 number, PortString& result) const;
//@method:PortString::Right
//@arg:number:uInt8
// How many character in the desired result string
//@arg:result:PortString reference
//@descr
// Returns the last "number" characters of the string as a result string.
//@endm
	void Right(uInt8 number, PortString& result)  const;
//@method:PortString::Mid
//@arg:start:uInt8
// The index (1-based) of the first character to return.
//@arg:number:uInt8
// How many character in the desired result string
//@arg:result:PortString reference
//@descr
// Returns a substring of the current value, starting at "start", and containing
// "number" characters.
//@endm
	void Mid(uInt8 start, uInt8 chars, PortString& result) const;
	
//@method:PortString::GetIntegerValue
//@return:sInt32
//  The value of the converted string
//@descr
//  Returns in "val" the numeric value of the string, or zero if it wasn't a string
//  at all (this uses atol internally, so it may return "partial" reads if the
//  string has leading digits).  Note that this checks the
//  current symbolics table in UtSymbolic for symbolic constant values, so some
//  "non-numeric" strings might translate correctly anyway.
//@endm
	sInt32	GetIntegerValue(void) const;
	
	
//@method:PortString::FindSubString
//@return:bool
// True if the substring exists in the current value.
//@arg:subString:PortString
// The substring to try and locate.
//@arg:location:uInt8 pointer
// Pointer to allocated character to hold resulting position.
//@descr
// If "substring" can be found as a substring of the current value,
// returns true and places the index of the first character of the
// substring into the uInt8 pointed to by "location."
//@endm
	bool FindSubString(PortString subString, uInt8* location);
//@method:PortString::ReplaceSubString
//@return:bool
// True if a replacement occurred
//@arg:subString:PortString
// The substring to try and locate.
//@arg:replace:PortString
// The string to replace the located one with.
//@descr
// If "substring" can be found as a substring of the current value,
// returns true and replaces all the characters of "substring" with
// the characters of "replace."   This replaces only the first instance
// of the substring.  To replace them all, loop while the return value
// is true.  "replace" can be empty, in which case this effectively 
// deletes the substring.  If the substring does not exist, returns 
// false and leaves the string unalterered.
//@endm
	bool ReplaceSubString(PortString subString, PortString replace);
	
//@method:PortString::RemoveWhiteSpace
//@descr
// Eliminates all spaces, tabs, and newlines from the current string.
//@endm
	void	RemoveWhiteSpace(void);
	
//@method:PortString::CountCharsMatching
//@arg:match:char
//@return:uInt16
//@descr
//  Returns the number of characters in the string that are equal to "match."
//@endm
	uInt16 	CountCharsMatching(char match);
	
//@method:PortString::Operator=
//@return:PortString reference
// A reference to the current string, as expected of operator=
//@arg:newValue:constCStringPtr, constPStringPtr, std::string, or PortString
//@descr
// Replace the string's value with the specified new one.
//@endm
	PortString& operator=(constPStringPtr theString) { SetValue(theString); return *this; }
	PortString& operator=(constCStringPtr theString) { SetValue(theString); return *this; }
	PortString& operator=(const PortString& theString) { SetValue(theString); return *this; }
	PortString& operator=(const std::string& theString) { SetValue(theString); return *this; }
	
//@method:PortString::Operator+=
//@arg:append_this:sInt32 or PortString
// Value to append
//@descr
// Appends the specified value, by way of the Append() method.
//@endm
	void operator+=(const PortString& addThis) { Append(addThis); }
	void operator+=(sInt32 addThis) { Append(addThis); }
		
//@method:PortString::Operator==
//@return:bool
// true if the strings are exactly equivalent (i.e. case-sensitive equiv).
//@arg:compare:PortString reference
//@descr
// Case-sensitively compares the specified string to the current value.
//@endm		
	bool operator==(const PortString& compare) const;
	bool operator!=(const PortString& compare) const { return !(*this == compare); }
	
//@method:PortString::CaseBlindEqual
//@return:bool
// true if the strings are equivalent except for case.
//@arg:compare:PortString reference
//@descr
// Case-insensitively compares the specified string to the current value.
//@endm		
	bool CaseBlindEqual(const PortString& compare) const;
	
//@method:PortString::MakeUpper
//@descr
//  Converts every alphabetic character to it's uppercase equivalent.
//@endm
	void	MakeUpper(void);
	

//@method:PortString::Operator[], GetNthChar
//@arg:n:int
//@descr
// Returns the "n"th character (1-based) of the string.  The operator[] 
// is defined for both unsigned char and const unsigned char, which occasionally
// causes difficulty with implicit conversions, so GetNthChar is provided as
// an alternative. 
//@endm		
	unsigned char& operator[](int i) { return iString[i]; }	
	const unsigned char& operator[](int i) const { return iString[i]; }
	char GetNthChar(int i) { return (char)iString[i]; }	
	void SetNthChar(int i, unsigned char x) { iString[i] = x; }

//@method:PortString::Conversion Operators
//@descr
// Conversion operators to const char*, const unsigned char*, char*, and unsigned char*
// are provided; you sometimes have to cast them explicitly to remove ambiguity.
//@end
	operator const char*(void) const { return (const char *)&iString[1]; }
	operator const unsigned char*(void) const { return iString; }
	operator char*(void) { return (char *)&iString[1]; }
	operator unsigned char*(void) { return iString; }
	
//@method:PortString::operator&lt;
//@return:bool
// true if the this string is less than "compare" in collation order (ASCII)
//@arg:compare:PortString reference
//@descr
// A thin shell over strcmp so that PortStrings are sortable.
//@endm		
	bool operator<(const PortString& other) const 
		{ return std::strcmp((const char*)this, (const char*)other) < 0; }
		
	
//@method:PortString::ResolveAfterPascalWrite
//@descr
// PortStrings aren't really meant to be written to natively.  If you pass one
// to a routine that writes to it believing it to be a Pascal string, you
// should call this routine afterward (which undoes the internal corruption that
// will be caused by the write.)  This is occasionally necessary on certain platforms.
//@endm		
	void ResolveAfterPascalWrite(void);
//@method:PortString::ResolveAfterCWrite
//@descr
// PortStrings aren't really meant to be written to natively.  If you pass one
// to a routine that writes to it believing it to be a C string, you
// should call this routine afterward (which undoes the internal corruption that
// will be caused by the write.)  This is occasionally necessary on certain platforms.
//@endm		
	void ResolveAfterCWrite(void);
	
//@method:PortString::GetLength
//@return:uInt8
// The number of characters in the string, not counting the trailing NULL.
//@descr
// Returns the length of the string.  This is the old name, this method
// is now called "Length()."
//@endm		
	uInt8 GetLength(void) const { return (const uInt8) iString[0]; }
//@method:PortString::Length
//@return:uInt8
// The number of characters in the string, not counting the trailing NULL.
//@descr
// Returns the length of the string.
//@endm		
	uInt8 Length(void) const { return GetLength(); } // New, better name
};

} // namespace PortableViews;

#endif