// A portable string class that knows how to deal with both
// C strings and Mac-centric Pascal Strings.

#include "UniversalSymbols.h"
#include "PortString.h"
#include <cstring>
#include <cctype>

namespace PortableViews {

// Got some local utilities here.
static long mystrlen(constCStringPtr aString)
	{
	long i = 0;
	while (aString[i] != kNullChar)
		i++;
	return i;
	}
	
static void mystrcpy(CStringPtr dest, constCStringPtr source)
	{
	long len = mystrlen(source);
	for (long i = 0; i <= len; i++)
		dest[i] = source[i];
	}

static void mystrncpy(CStringPtr dest, constCStringPtr source, sInt32 n)
	{
	long len = (mystrlen(source) < n)?mystrlen(source):n;
	for (long i = 0; i < len; i++)
		dest[i] = source[i];
	}

// Constructor for an empty string
PortString::PortString(void)
	{
	iString[0] = 0;
	iString[1] = kNullChar;
	}
	
void PortString::SetValue(const std::string& theString)
	{
	SetValue(theString.c_str());
	}
	
// Set value to a Pascal string
void PortString::SetValue(constPStringPtr theString)
	{
	std::memcpy(iString, theString, theString[0]+1);
	iString[theString[0]+1] = kNullChar;
	}
	
// Set value a C string
void PortString::SetValue(constCStringPtr theString)
	{
	if (mystrlen(theString) <= 255)
		{
		mystrcpy((CStringPtr)&iString[1], theString);
		iString[0] = (unsigned char) mystrlen(theString);
		}
	else
		{
		mystrncpy((CStringPtr)&iString[1], theString, 255);
		iString[0] = (unsigned char) 255;
		iString[256] = kNullChar;
		}
	}
	
std::string PortString::GetStdString(void)
	{
	std::string returnVal((char *)this);
	return returnVal;
	}
	
bool PortString::operator==(const PortString& compare) const
	{
	bool returnVal = true;
	if (iString[0] != compare.iString[0])
		returnVal = false;
	else
		{
		for (uInt8 i = 1; i <= iString[0]; i++)
			if (iString[i] != compare.iString[i])
				{
				returnVal = false;
				break;
				}
		}
	return returnVal;
	}	

bool PortString::CaseBlindEqual(const PortString& compare) const
	{
	bool returnVal = true;
	if (iString[0] != compare.iString[0])
		returnVal = false;
	else
		{
		for (uInt8 i = 1; i <= iString[0]; i++)
			if (std::toupper(iString[i]) != std::toupper(compare.iString[i]))
				{
				returnVal = false;
				break;
				}
		}
	return returnVal;
	}	

void	PortString::MakeUpper(void)
	{
	for (uInt8 i = 1; i <= iString[0]; i++)
		iString[i] = std::toupper(iString[i]);
	}
	
uInt16 	PortString::CountCharsMatching(char match)
	{
	uInt16 count = 0;
	for (uInt8 i = 1; i <= iString[0]; i++)
		if ((char)iString[i] == match)
			count++;
	return count;
	}


void PortString::Left(uInt8 number, PortString& result) const
	{
	// Don't return more than we've got.
	uInt8 realNum = std::min(number, iString[0]);
	result.iString[0] = realNum;
	result.iString[realNum+1] = 0;
	mystrncpy((CStringPtr)&result.iString[1], (CStringPtr)&iString[1], realNum);
	}
	
void PortString::Right(uInt8 number, PortString& result) const
	{
	PortString tmpResult;
	// Don't return more than we've got.
	uInt8 realNum = std::min(number, iString[0]);
	tmpResult.iString[0] = realNum;
	mystrncpy((CStringPtr)&tmpResult.iString[1], (CStringPtr)&iString[iString[0] - realNum + 1], realNum);
	tmpResult.iString[realNum+1] = 0;
	result = tmpResult;
	}
	
void PortString::Mid(uInt8 start, uInt8 chars, PortString& result) const
	{
	// Don't return more than we've got.
	if (start > iString[0])
		result.SetEmpty();
	else
		{
		uInt8 realNum = std::min((long)chars, (long)(iString[0] - start + 1));
		result.iString[0] = realNum;
		result.iString[realNum+1] = 0;
		mystrncpy((CStringPtr)&result.iString[1], (CStringPtr)&iString[start], realNum);
		}
	}

	
// Locates (only the first instance of) a substring.
bool PortString::FindSubString(PortString subString, uInt8* location)
	{
	char* loc = std::strstr((CStringPtr)&iString[1], subString);
	bool returnVal = (loc == NULL) ? false : true;
	if (returnVal)
		*location = (uInt8)(loc - ((char*)&iString[0]));
	return returnVal;
	}
	
// Replaces (only the first instance of) a substring of the string.
bool PortString::ReplaceSubString(PortString subString, PortString replace)
	{
	uInt8 start;
	bool returnVal = FindSubString(subString, &start);
	if (returnVal)
		{
		PortString tmp, tmp2;
		uInt8 sLen = subString.GetLength();
		uInt8 rLen = replace.GetLength();
		uInt8 oLen = GetLength();
		if (sLen == oLen)
			tmp.SetValue(replace);
		else if (start == 1)
			{
			tmp.SetValue(replace);
			Right(oLen - sLen, tmp2);
			tmp.Append(tmp2);
			}
		else if (((oLen - start)+1) == sLen)
			{
			Left(start-1, tmp);
			tmp.Append(replace);
			}
		else 
			{
			Left(start-1, tmp);
			tmp.Append(replace);
			Right(oLen - (start + sLen - 1), tmp2);
			tmp.Append(tmp2);
			}
		SetValue(tmp);
		}
	return returnVal;
	}

// Inefficiently eliminates all tabs and spaces from a string;
void	PortString::RemoveWhiteSpace(void)
	{
	ReplaceSubString(" ", "");
	ReplaceSubString("\t", "");
	}
	
// Set Value from a Resource
void PortString::SetValueFromResource(resID strRes)
	{
	GetPStringFromResource(iString, strRes);
	iString[iString[0]+1] = kNullChar;
	}

// Set Value from a Resource table
void PortString::SetValueFromResource(uInt8 strnRes, uInt8 index)
	{
	GetPStringFromResourceTable(iString, strnRes, index);
	iString[iString[0]+1] = kNullChar;
	}

	
// Append a Pascal String
void PortString::Append(constPStringPtr appendThis)
	{
	std::memcpy(&iString[iString[0]+1], &appendThis[1], appendThis[0]);
	iString[0] += appendThis[0];
	iString[iString[0]+1] = kNullChar;
	}
	
	
// Append a C string
void PortString::Append(constCStringPtr appendThis)
	{
	unsigned char currentLength = iString[0];
	if (mystrlen(appendThis) + currentLength <= 255)
		{
		for (short i = 0; i < mystrlen(appendThis); i++)
			iString[i + currentLength + 1] = appendThis[i];
		iString[0] += mystrlen(appendThis);
		iString[iString[0] + 1] = kNullChar;
		}
	else
		{
		for (short i = 0; currentLength + i < 255; i++)
			iString[i + currentLength + 1] = appendThis[i];
		iString[0] = 255;
		iString[256] = kNullChar;
		}
	}

// Assume the string's been written to as a Pascal String, fix the null character
void PortString::ResolveAfterPascalWrite(void)
	{
	iString[iString[0] + 1] = kNullChar;
	}
	
	
// Assume the string's been written to as a C String, fix the length byte
void PortString::ResolveAfterCWrite(void)
	{
	iString[0] = mystrlen((char*)&iString[1]);
	}
	
} // namespace PortableViews