/*
  This code demonstrates the creation of a ShapeStructure Graph from
  a MedialAxis Graph. It operates on a sample MedialAxis Graph
  provided by sampleMedialAxisGraph1(). The program prints the 
  resulting ShapeStructure Graph on stdout

  Compiles with MSVC 6.0 sp5 on Windows 2000
*/

#include <boost/config.hpp>
#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/depth_first_search.hpp> 

#include <iostream>
#include <string>
#include <stack>
#include <cmath>

using namespace boost;
using namespace std;



////////////////////////////////////////////////////////////////

// Declarations and definitions for the MedialAxisGraph
enum vertex_coord_t { vertex_coord  };
namespace boost {
  BOOST_INSTALL_PROPERTY(vertex, coord);
}

struct Point
{
  Point(double x = 0.0, double y = 0.0)
    :x_(x), y_(y)
  { }

  double x_,y_;
};

double mydistance(const Point& p1, const Point& p2)
{
  double dx = p1.x_-p2.x_;
  double dy = p1.y_-p2.y_;
  return sqrt(dx*dx + dy*dy);
}

typedef boost::property<vertex_coord_t, Point> MedialAxisGraphVertexProp;
typedef adjacency_list<vecS, vecS, undirectedS, 
                       MedialAxisGraphVertexProp, no_property> 
        MedialAxisGraph;


// Declarations and definitions for the ShapeStructureGraph

enum vertex_medaxis_vertex_t { vertex_medaxis_vertex };
namespace boost {
  BOOST_INSTALL_PROPERTY(vertex, medaxis_vertex);
}


enum edge_medaxis_walk_t { edge_medaxis_walk };
namespace boost {
  BOOST_INSTALL_PROPERTY(edge, medaxis_walk);
}

struct Walk
{
  Walk(double length = 0.0)
    : length_(length)
  { }

  double length_;
};


typedef boost::property<vertex_medaxis_vertex_t, 
          boost::graph_traits<MedialAxisGraph>::vertex_descriptor>
          ShapeStructureGraphVertexProp;

typedef boost::property<edge_medaxis_walk_t, Walk> 
          ShapeStructureGraphEdgeProp;


typedef adjacency_list<vecS, vecS, undirectedS, 
                       ShapeStructureGraphVertexProp, 
                       ShapeStructureGraphEdgeProp> 
        ShapeStructureGraph;

typedef boost::property_map<ShapeStructureGraph, 
        vertex_medaxis_vertex_t>::type MedialAxisVertexIdPropMap;


// Declarations and definitions for Depth-First-Search visitors


struct InitVertex : public base_visitor<InitVertex> {
  typedef on_initialize_vertex event_filter;
  template <class Vertex, class Graph>
  void operator()(Vertex v, Graph&) {
    std::cout << "InitVertex: " << v << std::endl;
  }
};
struct StartVertex : public base_visitor<StartVertex> {
  typedef on_start_vertex event_filter;
  template <class Vertex, class Graph>
  void operator()(Vertex v, Graph&) {
    std::cout << "StartVertex: " << v << std::endl;
  }
};


struct DiscoverVertex : public base_visitor<DiscoverVertex> {
  typedef on_discover_vertex event_filter;
  template <class Vertex, class Graph>
  void operator()(Vertex mVertex, Graph& mGraph) 
  {
    Point p = boost::get(boost::get(vertex_coord_t(),mGraph),mVertex);
    
    dist_ += mydistance(p,pLast_->top());
    pLast_->pop();
    pLast_->push(p);

    if(out_degree(mVertex,mGraph) == 2)
      return;

    // The rest of the function processes
    // junctions (out_degree > 2) or leaf vertices 
    // (out_degree == 1)
    graph_traits<ShapeStructureGraph>::vertex_descriptor sVertex;
    sVertex = boost::add_vertex(*sGraph_);
    boost::put(vertex_medaxis_vertex_t(),
      *sGraph_,
      sVertex,
      mVertex);
    
    if(boost::num_vertices(*sGraph_) > 1)
    {
      graph_traits<ShapeStructureGraph>::edge_descriptor sEdge;
      bool duplicate;
      boost::tie(sEdge,duplicate) = 
        boost::add_edge(sVertex,vLast_->top(),*sGraph_);
      boost::put(edge_medaxis_walk_t(),
        *sGraph_,
        sEdge,
        Walk(dist_));
    }
    
    dist_ = 0;
    
    vLast_->pop();
    pLast_->pop();
    for(int i = (out_degree(mVertex,mGraph) -1); i > 0; --i)
    {
      vLast_->push(sVertex);
      pLast_->push(p);
    }
  }

  DiscoverVertex(
                 std::stack<graph_traits<ShapeStructureGraph>::vertex_descriptor>* ln = 0,
                 std::stack<Point>* pLast = 0,
                 ShapeStructureGraph* g = 0)
    :dist_(0), vLast_(ln), pLast_(pLast), sGraph_(g)
  { 
    
  }

  double dist_;
  std::stack<graph_traits<ShapeStructureGraph>::vertex_descriptor>* vLast_;
  std::stack<Point>* pLast_;
  ShapeStructureGraph* sGraph_;
};


struct FinishVertex : public base_visitor<FinishVertex> {
  typedef on_finish_vertex event_filter;
  template <class Vertex, class Graph>
  void operator()(Vertex v, Graph&) {
    std::cout << "FinishVertex: " << v << std::endl;
  }
};
struct ExamineEdge : public base_visitor<ExamineEdge> {
  typedef on_examine_edge event_filter;
  template <class Edge, class Graph>
  void operator()(Edge e, Graph&) {
    std::cout << "ExamineEdge: " << e << std::endl;
  }
};
struct TreeEdge : public base_visitor<TreeEdge> {
  typedef on_tree_edge event_filter;
  template <class Edge, class Graph>
  void operator()(Edge e, Graph&) {
    std::cout << "TreeEdge: " << e << std::endl;
  }
};
struct BackEdge : public base_visitor<BackEdge> {
  typedef on_back_edge event_filter;
  template <class Edge, class Graph>
  void operator()(Edge e, Graph&) {
    std::cout << "BackEdge: " << e << std::endl;
  }
};
struct FwdOrCrossEdge : public base_visitor<FwdOrCrossEdge> {
  typedef on_forward_or_cross_edge event_filter;
  template <class Edge, class Graph>
  void operator()(Edge e, Graph&) {
    std::cout << "FwdOrCrossEdge: " << e << std::endl;
  }
};

void sampleMedialAxisGraph1
 (MedialAxisGraph& g, 
  graph_traits<MedialAxisGraph>::vertex_descriptor& dfsStart)
{
  const int numVertices = 12;
  g = MedialAxisGraph(numVertices);

  add_edge(11,0,g);
  add_edge(0,1,g);
  add_edge(1,2,g);
  add_edge(2,3,g);
  add_edge(3,10,g);
  add_edge(1,5,g);
  add_edge(5,8,g);
  add_edge(8,9,g);
  add_edge(0,4,g);
  add_edge(4,6,g);
  add_edge(6,7,g);

  boost::put(vertex_coord_t(),g,11,Point(-1,0));
  boost::put(vertex_coord_t(),g, 0,Point(0,0));
  boost::put(vertex_coord_t(),g, 1,Point(1,0));
  boost::put(vertex_coord_t(),g, 2,Point(2,0));
  boost::put(vertex_coord_t(),g, 3,Point(3,0));
  boost::put(vertex_coord_t(),g,10,Point(4,0));
  boost::put(vertex_coord_t(),g, 5,Point(1,-1));
  boost::put(vertex_coord_t(),g, 8,Point(1,-2));
  boost::put(vertex_coord_t(),g, 9,Point(1,-3));
  boost::put(vertex_coord_t(),g, 4,Point(0,-1));
  boost::put(vertex_coord_t(),g, 6,Point(0,-2));
  boost::put(vertex_coord_t(),g, 7,Point(0,-3));

  // start vertex must be a leaf
  dfsStart = 11; 
}


void medialAxisAnalysis(void)
{
  MedialAxisGraph medAxis;
  graph_traits<MedialAxisGraph>::vertex_descriptor dfsStart;
  sampleMedialAxisGraph1(medAxis, dfsStart);
  ShapeStructureGraph shapeStruct;

  std::vector<default_color_type> color(num_vertices(medAxis));
  std::stack<graph_traits<ShapeStructureGraph>::vertex_descriptor> vLast;
  std::stack<Point> pLast;


  // adjacency_list<vecS,vecS> uses 0-based indices
  // as vertex descriptors
  vLast.push(static_cast<graph_traits<ShapeStructureGraph>::vertex_descriptor>(0));
  vLast.push(static_cast<graph_traits<ShapeStructureGraph>::vertex_descriptor>(0));

  boost::property_map<MedialAxisGraph, vertex_coord_t>::type coords =
    boost::get(vertex_coord_t(),medAxis);
  Point p2 = boost::get(coords,dfsStart); 
  pLast.push(p2);
  pLast.push(p2);

  dfs_visitor<pair<DiscoverVertex,null_visitor> > vis =
    std::make_pair(DiscoverVertex(&vLast,&pLast,&shapeStruct),
              null_visitor());

  depth_first_search(medAxis, vis, &color[0], dfsStart);
  

  std::cout << "=============================================" << std::endl;

  std::cout << "Printing ShapeStructureGraph: " << std::endl;

  boost::property_map<ShapeStructureGraph,edge_medaxis_walk_t>::type walks =
    boost::get(edge_medaxis_walk_t(),shapeStruct);
  boost::graph_traits<ShapeStructureGraph>::edge_iterator it1, it2;
  boost::tie(it1,it2) = edges(shapeStruct);
  for(; it1 != it2; ++it1)
  {
    boost::graph_traits<ShapeStructureGraph>::vertex_descriptor v1,v2;
    v1 = boost::source(*it1,shapeStruct);
    v2 = boost::target(*it1,shapeStruct);
    std::cout 
      << *it1 
      << " maps to medaix vertices ("

      <<  boost::get(boost::get(vertex_medaxis_vertex_t(),shapeStruct), 
                                boost::source(*it1,shapeStruct))
      << ", "
      <<  boost::get(boost::get(vertex_medaxis_vertex_t(),shapeStruct), 
                                boost::target(*it1,shapeStruct))

      << ")"
      << "\twalk length =  "
      << boost::get(walks,*it1).length_
      << std::endl;
  }

  std::cout << "=============================================" << std::endl;
}


int main(int argc, char* argv[])
{
  medialAxisAnalysis();
  return 0;
};

